package main.experimental.ui.experdialog;


import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Random;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.DefaultTableModel;

import main.phrasemodel.management.model.ExperModel;
import model.Experiment;
import model.ExperimentsTableModel;
import model.Questionnaire;
import model.QuestionnairesTableModel;
import tools.util.Constant;

@SuppressWarnings("serial")
public class ExperiencePane extends JPanel {
	
	private JTable expTable = new JTable(ExperimentsTableModel.getInstance());
	private JTable questTable;
	private ExperModel mtModel;	

	private JButton newBtn = new JButton(Constant.EXP_EXP_INSERT);
	private JButton editBtn = new JButton(Constant.EXP_EXP_EDIT);
	private JButton seeBtn = new JButton(Constant.EXP_EXP_SEE); // move to Question Pane
	private JButton analysisBtn = new JButton(Constant.EXP_EXP_ANALYSIS);
	private JButton generateDataBtn = new JButton(Constant.GENERATE_R_DOT_DATA);
	private JButton delBtn = new JButton(Constant.EXP_EXP_DELETE);
	private ExperiencePane ep;

	public ExperiencePane(ExperModel mtModel, JTable questTable) {
		this.questTable = questTable;
		this.mtModel = mtModel;
		ep = this;
		
		JPanel btnPanel = new JPanel();
		btnPanel.setLayout(new FlowLayout());
		btnPanel.add(newBtn);
		btnPanel.add(editBtn);
		btnPanel.add(seeBtn);
		btnPanel.add(analysisBtn);
		btnPanel.add(generateDataBtn);
		btnPanel.add(delBtn);

		JScrollPane jsExpTable = new JScrollPane(expTable);

		this.setLayout(new BorderLayout());
		this.add(btnPanel, BorderLayout.NORTH);
		this.add(jsExpTable , BorderLayout.CENTER);
		this.setBorder(BorderFactory.createTitledBorder(Constant.EXP_EXP));
		
		buildBtnListener();
	}
	
	public void buildBtnListener(){
		expTable.getSelectionModel().addListSelectionListener(new ListSelectionListener() {
			@Override
			public void valueChanged(ListSelectionEvent e) {
				int row = expTable.getSelectedRow();
				if(row != -1){
					Experiment exp = ExperimentsTableModel.getInstance().getAllExperiments().get(row);
					QuestionnairesTableModel.getInstance(exp).fireTableDataChanged();
					questTable.setModel(QuestionnairesTableModel.getInstance(exp));
					//鳥寫法  不動太多結構下的不得以方法
					if(Constant.ID.equals(questTable.getColumnName(1))){
						questTable.removeColumn(questTable.getColumn(Constant.ID));
						questTable.removeColumn(questTable.getColumn(Constant.NUMOF));
					}
				} else {
					questTable.setModel(new DefaultTableModel());
				}
			}
		});

		newBtn.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				new NewExperDialog(ep, mtModel, expTable);
			}
		});
		
		editBtn.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				int row = expTable.getSelectedRow();
				if(-1 == row){
					JOptionPane.showMessageDialog(ep, Constant.ERROR_MSG_NOT_SELECT, Constant.ERROR_MSG_ERROR, JOptionPane.ERROR_MESSAGE);
				}else{
					new EditExperDialog(ep, ExperimentsTableModel.getInstance(), row);
				}
			}
		});
		
		delBtn.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				int[] rows = expTable.getSelectedRows();
				if(rows.length == 0){
					JOptionPane.showMessageDialog(ep, Constant.ERROR_MSG_NOT_SELECT, Constant.ERROR_MSG_ERROR, JOptionPane.ERROR_MESSAGE);
				}else{
					int choose = JOptionPane.showConfirmDialog(ep, Constant.MSG_IS_DELETE, Constant.MSG_NOTE, JOptionPane.YES_NO_OPTION);
					if( choose == JOptionPane.YES_OPTION ){
						ExperimentsTableModel instance = ExperimentsTableModel.getInstance();
						ArrayList<Experiment> toBeRemoved = new ArrayList<Experiment>();
						for (int i: rows) {
							toBeRemoved.add(instance.getAllExperiments().get(i));
						}
						for (Experiment e: toBeRemoved) {
							instance.getAllExperiments().remove(e);
							e.delete();
						}
						instance.fireTableDataChanged();
					}
				}
			}
		});
		
		seeBtn.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				if(-1 == expTable.getSelectedRow()){
					JOptionPane.showMessageDialog(ep, Constant.ERROR_MSG_NOT_SELECT, Constant.ERROR_MSG_ERROR, JOptionPane.ERROR_MESSAGE);
				}else{
					// select one questionnaire randomly
					int row = expTable.getSelectedRow();
					Experiment experiment = ExperimentsTableModel.getInstance().getAllExperiments().get(row);
					ArrayList<Questionnaire> qs = experiment.getQuestionnaires();
					Random rand = new Random();
					if(0 == qs.size()) {
						try {
							Questionnaire q = new Questionnaire(experiment);
							new QueSampleDialog(ep, q);
							q.delete();
						} catch (RuntimeException re) {
							re.printStackTrace();
							JOptionPane.showConfirmDialog(ep, "exception on generating a new questionnaire. one possibility is the refered factor id is no longer existent.");
						}
					} else {
						new QueSampleDialog(ep, qs.get(rand.nextInt(qs.size())));
					}
				}
			}
		});
		
		analysisBtn.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				int row = expTable.getSelectedRow();
				if(-1 == row){
					JOptionPane.showMessageDialog(ep, Constant.ERROR_MSG_NOT_SELECT, Constant.ERROR_MSG_ERROR, JOptionPane.ERROR_MESSAGE);
				}else{
					Experiment exp = ExperimentsTableModel.getInstance().getAllExperiments().get(row);
					
					int i = 2;
					for(Questionnaire q : exp.getQuestionnaires()){
						if(q.getState().equals(Questionnaire.State.COMPLETED)){
							i--;
						}
					}
					if(i <= 0){
						exp.analyze();
					}else{
						JOptionPane.showMessageDialog(ep, Constant.WARRING_MSG_MORE_THAN_TWO, Constant.WARNING_MSG_WARNING, JOptionPane.WARNING_MESSAGE);
					}
					
				}
				// invoke r and generate the image
				// load the image
			}
		});
		
		generateDataBtn.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				int row = expTable.getSelectedRow();
				if(-1 == row){
					JOptionPane.showMessageDialog(ep, Constant.ERROR_MSG_NOT_SELECT, Constant.ERROR_MSG_ERROR, JOptionPane.ERROR_MESSAGE);
				}else{
					JFileChooser saveFileDialog = new JFileChooser();
					int result = saveFileDialog.showSaveDialog(null);
					if(JFileChooser.APPROVE_OPTION == result){
						Experiment exp = ExperimentsTableModel.getInstance().getAllExperiments().get(row);
						exp.generateRData(saveFileDialog.getSelectedFile().getPath());
					}
				}
			}
		});
	}

	public JTable getTable() {
		return expTable;
	}
}
