package main.experimental.ui.innerQuestionPane;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.print.PageFormat;
import java.awt.print.PrinterJob;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import main.experimental.ui.executeQuestionnaire.ExecuteQuestion;
import main.experimental.ui.innerQuestionPane.Refill.Refill;
import model.Experiment;
import model.ExperimentsTableModel;
import model.LittleOne;
import model.Question;
import model.Questionnaire;
import model.QuestionnairesTableModel;
import model.SettingModel;

import org.springframework.validation.BindException;

import tools.util.Constant;
import tools.util.FilePrinter;
import tools.util.GsonGenerator;
import tools.validation.MailSettingValidation;
import vo.MailInfo;
import vo.MailSettingData;
import vo.SurveyResult;
import vo.UploadQuestionaireData;

import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;

@SuppressWarnings("serial")
public class InnerQuestionPane extends JPanel {
	
	private JButton newBtn = new JButton(Constant.EXP_SURVEY_INSERT);
	private JButton editBtn = new JButton(Constant.EXP_SURVEY_EDIT);
	private JButton execBtn = new JButton(Constant.EXP_SURVEY_EXECUTE);
	private JButton mailBtn = new JButton(Constant.EXP_SURVEY_MAIL);
	private JButton dlBtn = new JButton(Constant.EXP_SURVEY_DOWNLOAD);
	private JButton printBtn = new JButton(Constant.EXP_SURVEY_PRINT);
	private JButton refillBtn = new JButton(Constant.EXP_SURVEY_REFILL);
	private JButton delBtn = new JButton(Constant.EXP_SURVEY_DELETE);
	
	private JTable expTable;
	private JTable queTable = new JTable();
	private InnerQuestionPane iq;
	private HttpURLConnection conn;
	private ListSelectionModel selectionModel;
	private boolean isPrint = false;

	public InnerQuestionPane() {
		iq = this;

		JPanel btnPanel = new JPanel();
		btnPanel.setLayout(new FlowLayout());
		btnPanel.add(newBtn);
		btnPanel.add(editBtn);
		btnPanel.add(execBtn);
		btnPanel.add(mailBtn);
		btnPanel.add(dlBtn);
		btnPanel.add(printBtn);
		btnPanel.add(refillBtn);
		btnPanel.add(delBtn);

		printBtn.setEnabled(false);
		refillBtn.setEnabled(false);
		
		JScrollPane jsQueTable = new JScrollPane(queTable);
		selectionModel = queTable.getSelectionModel();
		buildListener();

		this.setLayout(new BorderLayout());
		this.add(btnPanel, BorderLayout.NORTH);
		this.add(jsQueTable , BorderLayout.CENTER);
		this.setBorder(BorderFactory.createTitledBorder(Constant.EXP_SURVEY));
		//this.pack();
	}
	
	public void setExpTable(JTable tbl) {
		expTable = tbl;
	}
	
	private void buildListener(){
		//Table的Select控制按鈕的Enable & Disable
		selectionModel.addListSelectionListener(new ListSelectionListener() {
			public void valueChanged(ListSelectionEvent e) {
				ListSelectionModel lsm = (ListSelectionModel) e.getSource();
				//若沒有選時，按鈕的狀態
				if(lsm.isSelectionEmpty()){
					printBtn.setEnabled(false);
					refillBtn.setEnabled(false);
				} else {
					int[] rows = queTable.getSelectedRows();
					//超過一筆
					if(rows.length > 1){
						printBtn.setEnabled(false);
						refillBtn.setEnabled(false);
						return;
					}
					Questionnaire q = ((QuestionnairesTableModel)queTable.getModel()).getExperiment().getQuestionnaires().get(rows[0]);
					//狀態檢查
					if (Questionnaire.State.NEW.equals(q.getState())) {
						printBtn.setEnabled(true);
//						refillBtn.setEnabled(false);
						refillBtn.setEnabled(true);
					} else if(Questionnaire.State.PRINT.equals(q.getState())){
						printBtn.setEnabled(false);
						refillBtn.setEnabled(true);
					} else {
						printBtn.setEnabled(false);
						refillBtn.setEnabled(false);
					}
				}
			}
		});
		
		buildBtnListener();
	}
	
	private void buildBtnListener() {
		// 新增問卷
		newBtn.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				int row = expTable.getSelectedRow();
				if (-1 == row) {
					// no experiment selected
					return;
				}
				ExperimentsTableModel model = (ExperimentsTableModel) expTable.getModel();
				Experiment exp = model.getAllExperiments().get(row);
				try {
					new Questionnaire(exp);
					ExperimentsTableModel.getInstance().fireTableRowsUpdated(row, row);
					QuestionnairesTableModel.getInstance(exp).fireTableRowsInserted(exp.getQuestionnaires().size() - 1, exp.getQuestionnaires().size() - 1);
				} catch (RuntimeException re) {
					re.printStackTrace();
					JOptionPane.showConfirmDialog(iq, "exception on generating a new questionnaire. one possibility is the refered factor id is no longer existent.");
				}
			}
		});
		editBtn.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				if(-1 == queTable.getSelectedRow()){
					JOptionPane.showMessageDialog(iq, Constant.ERROR_MSG_NOT_SELECT, Constant.ERROR_MSG_ERROR, JOptionPane.ERROR_MESSAGE);
				}else{
					new EditQueDialog(iq,(QuestionnairesTableModel)queTable.getModel(),queTable.getSelectedRow());
				}
			}
		});
		delBtn.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				int[] rows = queTable.getSelectedRows();
				if(rows.length == 0){
					JOptionPane.showMessageDialog(iq, Constant.ERROR_MSG_NOT_SELECT, Constant.ERROR_MSG_ERROR, JOptionPane.ERROR_MESSAGE);
				}else{
					int choose = JOptionPane.showConfirmDialog(iq, Constant.MSG_IS_DELETE, Constant.MSG_NOTE, JOptionPane.YES_NO_OPTION);
					if( choose == JOptionPane.YES_OPTION ){
						QuestionnairesTableModel model = (QuestionnairesTableModel)queTable.getModel();
						ArrayList<Questionnaire> qs = model.getExperiment().getQuestionnaires();
						ArrayList<Questionnaire> toBeRemoved = new ArrayList<Questionnaire>();
						for (int i: rows) {
							toBeRemoved.add(qs.get(i));
						}
						for (Questionnaire q: toBeRemoved) {
							q.delete();
						}
						model.fireTableDataChanged();
					}
				}
			}
		});
		execBtn.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				int row = queTable.getSelectedRow();
				if(-1 == row){
					JOptionPane.showMessageDialog(iq, Constant.ERROR_MSG_NOT_SELECT, Constant.ERROR_MSG_ERROR, JOptionPane.ERROR_MESSAGE);
				
				}else{
					// get questionnaire
					Questionnaire q = ((QuestionnairesTableModel)queTable.getModel()).getExperiment().getQuestionnaires().get(row);
					// check status
					if (!Questionnaire.State.NEW.equals(q.getState())) {
						JOptionPane.showMessageDialog(iq, Constant.EXP_SURVEY_MSG_STATE_IS+ q.getState() +Constant.EXP_SURVEY_MSG_CAN_NOT_EXECUTE, Constant.ERROR_MSG_ERROR, JOptionPane.ERROR_MESSAGE);
					} else {
						//new ExecuteQuestion(qtModel, expModel, qtModel.getValueAt(queTable.getSelectedRow(), 0)+"");
						new ExecuteQuestion(iq, (QuestionnairesTableModel)queTable.getModel(),row);
					}
				}
			}
		});
		mailBtn.addActionListener(new ActionListener() {
			//按下按鈕後，若使用者選取多筆資料，要告知只會寄送第一筆資料或〔告知使用者只能選一筆〕。
			public void actionPerformed(ActionEvent arg0) {
				
				//沒有選擇的檢查
				if(-1 == queTable.getSelectedRow()){
					JOptionPane.showMessageDialog(iq, Constant.ERROR_MSG_NOT_SELECT, Constant.ERROR_MSG_ERROR, JOptionPane.ERROR_MESSAGE);
					return;
				}
				int[] rows = queTable.getSelectedRows();
				//選超過一筆的檢查
				if(rows.length > 1){
					JOptionPane.showMessageDialog(iq, Constant.ERROR_MSG_ONLY_ONE, Constant.ERROR_MSG_ERROR, JOptionPane.ERROR_MESSAGE);
					return;
				}
				int row = queTable.getSelectedRow();
				Questionnaire q = ((QuestionnairesTableModel)queTable.getModel()).getExperiment().getQuestionnaires().get(row);
				//此筆資料的狀態檢查
				if (!Questionnaire.State.NEW.equals(q.getState())) {
					JOptionPane.showMessageDialog(iq, Constant.EXP_SURVEY_MSG_STATE_IS+ q.getState() +Constant.EXP_SURVEY_MSG_CAN_NOT_EXECUTE, Constant.ERROR_MSG_ERROR, JOptionPane.ERROR_MESSAGE);
					return;
				}
				
				SettingModel sModel = new SettingModel();
				String server = sModel.select(Constant.SERVER).getValue();
				if("".equals(server)){
					JOptionPane.showMessageDialog(iq, "Server "+Constant.MSG_NOT_SET, Constant.ERROR_MSG_ERROR, JOptionPane.ERROR_MESSAGE);
					return;
				}
				
				MailSettingValidation validation = new MailSettingValidation();
				MailSettingData settingData = new MailSettingData();
				settingData.setName(sModel.select(Constant.MAIL_USERNAME).getValue());
				settingData.setSender(sModel.select(Constant.MAIL).getValue());
				settingData.setSmtp(sModel.select(Constant.SMTP).getValue());
				settingData.setPort(sModel.select(Constant.PORT).getValue());
				settingData.setSsl(sModel.select(Constant.SSL).getValue());
				settingData.setContent(sModel.select(Constant.CONTENT).getValue());
				BindException errors = new BindException(settingData, "MailSettingData");
				validation.validate(settingData, errors);
				if(errors.getErrorCount()>0){
					JOptionPane.showMessageDialog(iq, "Mail "+Constant.MSG_NOT_SET, Constant.ERROR_MSG_ERROR, JOptionPane.ERROR_MESSAGE);
					return;
				}
				
				LittleOne l = new LittleOne();
				UploadQuestionaireData data = l.getUploadQuestionaireData(Integer.parseInt(((QuestionnairesTableModel) queTable.getModel()).getId(row).toString()));
				GsonGenerator gsonGenerator = new GsonGenerator();
				gsonGenerator.generateDBJson(data);
				
				try {
					postJsonToServer(gsonGenerator.getJson(), server + Constant.GWT_URL);
					String[] msg = getResponse(conn).split("&");
					String uuid = msg[0];
					
					if(!"Error".equals(uuid)){
						MailInfo info = new MailInfo();
						info.setContext(settingData.getContent().replaceAll("\\"+Constant.LINK_REPLACEMENT, server + msg[1]));
						info.setUuid(uuid);
						info.setName(settingData.getName());
						info.setPort(settingData.getPort());
						info.setSender(settingData.getSender());
						info.setSmtp(settingData.getSmtp());
						if("1".equals(settingData.getSsl())){
							info.setSsl(true);
						}else{
							info.setSsl(false);
						}
						new MailPostDialog(queTable, info);
					}else{
						System.out.println("Error");
					}
				} catch (IOException e) {
					e.printStackTrace();
					System.out.println("Error");
					JOptionPane.showMessageDialog(iq, Constant.EXP_SURVEY_MSG_CONNECT_ERROR, Constant.ERROR_MSG_ERROR, JOptionPane.ERROR_MESSAGE);
				}
			}
		});
		
		dlBtn.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				
				SettingModel sModel = new SettingModel();
				String server = sModel.select(Constant.SERVER).getValue();
				if("".equals(server)){
					JOptionPane.showMessageDialog(iq, "Server "+Constant.MSG_NOT_SET, Constant.ERROR_MSG_ERROR, JOptionPane.ERROR_MESSAGE);
					return;
				}
				
				//取得所有的問卷的uuid		一組資料
				LittleOne l = new LittleOne();
				List<String> canDl = l.getUid();
				GsonGenerator gsonGenerator = new GsonGenerator();
				gsonGenerator.generateUUIDJson(canDl);
				
				try {
					postJsonToServer(gsonGenerator.getJson(), server + Constant.GWT_DOWNLOAD_SURVEY_RESULT);
					
					Gson gson = new Gson();
					List<SurveyResult> data = gson.fromJson(getResponse(conn), new TypeToken<List<SurveyResult>>(){}.getType());
					l.saveDownloadResult(data);
					ExperimentsTableModel.getInstance().fireTableDataChanged();
					JOptionPane.showMessageDialog(iq, Constant.MSG_DL_FINISH);
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		});
		printBtn.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				int row = queTable.getSelectedRow();
				Questionnaire q = ((QuestionnairesTableModel)queTable.getModel()).getExperiment().getQuestionnaires().get(row);
				
				new FilePrinter(createTXTTempFile(), iq);
				if(isPrint()) {
					q.setState(Questionnaire.State.PRINT);
					q.setNotes("Print:" + q.getId());
					((QuestionnairesTableModel)queTable.getModel()).fireTableRowsUpdated(row, row);
					printBtn.setEnabled(false);
				}
			}
		});
		refillBtn.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				//詢問使用者要列印還是回填問卷
//				Object[] options = { Constant.EXP_SURVEY_PRINT, Constant.EXP_SURVEY_REFILL };
//				int option = JOptionPane.showOptionDialog(null, Constant.WARRING_MSG_WANT_TO_DO, "Warning",
//				            JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE,
//				            null, options, options[0]);
//				//if 要列印的話就是call 列印的模組
//				//else show refill panel
//				if(option == 0){
//					
//				} else if(option == 1) {
					int row = queTable.getSelectedRow();
					new Refill(iq, (QuestionnairesTableModel)queTable.getModel(), row);
//				}
			}
		});
	}
	
	//XXX 可拉出去做一個class
	
	/**
	 * 透過 POST 方式將 Json List 傳到 Server 端。
	 * @param jsonList  Json ArrayList
	 * @return          uuid & 實驗網址。
	 * @throws IOException
	 */
	protected void postJsonToServer(ArrayList<String> jsonList, String urlStr) throws IOException{
		URL url = new URL(urlStr);
		conn = (HttpURLConnection) url.openConnection();
		conn.setDoOutput(true); 
		conn.setDoInput(true);
		conn.setRequestMethod("POST"); 
		conn.setUseCaches(false);
		conn.setReadTimeout(60000);
		
		Iterator<String> iterator = jsonList.iterator();
		StringBuffer json = new StringBuffer("");
		while(iterator.hasNext()){
			json.append(iterator.next() + "&");
		}
		
		conn.getOutputStream().write(json.toString().getBytes("utf-8"));
		conn.getOutputStream().flush();
		conn.getOutputStream().close();
	}
	
	protected String getResponse(HttpURLConnection conn) throws IOException{
		String response = "";
		if(conn.getResponseCode() == 200){
			
			BufferedReader br = new BufferedReader(new InputStreamReader(conn.getInputStream(), "UTF-8"));
			response = br.readLine();
			br.close();
			conn.disconnect();
		}
		return response;
	}

	public JTable getTable() {
		return queTable;
	}
	
	private File createTXTTempFile() {
		int row = queTable.getSelectedRow();
		Questionnaire q = ((QuestionnairesTableModel)queTable.getModel()).getExperiment().getQuestionnaires().get(row);
		File tmpFile = null;
		try {
			tmpFile = new File("print.txt");
			if (tmpFile.exists()) {
				tmpFile.delete();
			}
			BufferedWriter fw = new BufferedWriter(new FileWriter(tmpFile));
			
			PrinterJob pj = PrinterJob.getPrinterJob();
			PageFormat pageFormat = pj.defaultPage();
			
			String newLine = System.getProperty("line.separator");
			//實驗編號(實驗標題 + 日期) + 問卷編號(備註)
			fw.write(q.getExperiment().getTitle() + "-" + q.getExperiment().getDate() + "-" + "Print:" + q.getId() + newLine + newLine);
			fw.write("感謝您參加本實驗：" + newLine + newLine);
			fw.write(q.getExperiment().getGuideGuide() + newLine + newLine + newLine);
			fw.write("受試者編號：____" + newLine + newLine);
			fw.newLine();
			int i = 0;
			
			for(Question aQuestion : q.getQuestions()) {
				i++;
				String anwers = "() " + q.getExperiment().getPositive() + " () " + q.getExperiment().getNegative() + "  (" + i + ") ";
				//判斷列印的最大寬度，這邊不考慮選項過長的情況。
				int limit = (int)pageFormat.getWidth() / 12 - anwers.length();
				if (aQuestion.getQuestion().length() + anwers.length() > limit) {
					int len = 0;
					
					while (len < aQuestion.getQuestion().length()) {
						if(len == 0) {
							fw.write(anwers);
						} else {
							//補空白
							fw.write(String.format("%1$" + anwers.length() + "s", " "));
						}
						if(len + limit - 1 >= aQuestion.getQuestion().length()) {
							fw.write(aQuestion.getQuestion().substring(len, aQuestion.getQuestion().length()));
						} else {
							fw.write(aQuestion.getQuestion().substring(len, len + limit - 1));
						}
						
						len += limit - 1;
					}
				} else {
					fw.write(anwers + aQuestion.getQuestion());
				}
				//如果是最後一個問題就不加上換行字元，以免列印時造成最後一個換行字元在分頁時，
				//剛好成為最後一頁的第一行，造成多印一張空白頁。
				if( i < q.getQuestions().size()) {
					fw.newLine();
				}
			}
			fw.flush();
			fw.close();
		} catch (FileNotFoundException ex) {
			ex.printStackTrace();
		} catch (IOException ex) {
			ex.printStackTrace();
		}
		
		return tmpFile;
	}

	public void setPrint(boolean isPrint) {
		this.isPrint = isPrint;
	}

	public boolean isPrint() {
		return isPrint;
	}
}
