package main.experimental.ui.innerQuestionPane;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Iterator;
import java.util.List;

import javax.mail.MessagingException;
import javax.mail.internet.AddressException;
import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPasswordField;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.JTextField;

import model.LittleOne;
import model.Questionnaire;
import model.QuestionnairesTableModel;

import org.springframework.validation.BindException;
import org.springframework.validation.BindingResult;
import org.springframework.validation.ObjectError;

import tools.db.DAO;
import tools.util.Constant;
import tools.util.ScreenSet;
import tools.validation.MailValidation;
import vo.MailInfo;

public class MailPostDialog extends JDialog implements ActionListener {
	private static final long serialVersionUID = -1872427483226041668L;
	
	private JLabel lbPass;
	private JLabel lbReciver;
	private JLabel lbTitle;
	
	private JPasswordField tfPass;
	private JTextField tfReciver;
	private JTextField tfTitle;
	
	private JTextArea taContent;
	private JScrollPane scroll;
	
	private JButton sendBtn;
	private JButton cancelBtn;
	
	private MailInfo info;
	private JTable queTable;
	
	public MailPostDialog(JTable queTable, MailInfo info) {
		this.info = info;
		this.queTable = queTable;
		
		lbPass = new JLabel(Constant.PASSWORD + ":");
		lbReciver = new JLabel(Constant.TO + ":");
		lbTitle = new JLabel(Constant.SUBJECT + ":");
		
		tfPass = new JPasswordField(10);
		tfReciver = new JTextField(10);
		tfTitle = new JTextField(10);
		
		taContent = new JTextArea(5,20);
		scroll = new JScrollPane(taContent);
		
		sendBtn = new JButton(Constant.SEND);
		sendBtn.addActionListener(this);
		
		cancelBtn = new JButton(Constant.CANCEL);
		cancelBtn.addActionListener(this);
		
		taContent.append(info.getContext());
		
		Box box = Box.createVerticalBox();
		Box b1 = Box.createHorizontalBox();
		Box b2 = Box.createHorizontalBox();
		Box b3 = Box.createHorizontalBox();
		Box b4 = Box.createHorizontalBox();
		
		b1.add(lbPass);
		b1.add(tfPass);
		
		b2.add(lbReciver);
		b2.add(tfReciver);
		
		b3.add(lbTitle);
		b3.add(tfTitle);
		
		b4.add(sendBtn);
		b4.add(cancelBtn);
		
		box.add(b1);
		box.add(b2);
		box.add(b3);
		box.add(scroll);
		box.add(b4);
		add(box);

		this.getRootPane().setDefaultButton(sendBtn);
		this.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
		this.setResizable(false);
		this.pack();
		this.setModal(true);
		ScreenSet.setWidowCenter(this);
		this.setVisible(true);
	}
	
	public void actionPerformed(ActionEvent e) {
		JButton btn = (JButton)e.getSource();
		if(sendBtn.equals(btn)){
			setTextFieldsValue();
			
			MailValidation validation = new MailValidation();
			BindException errors = new BindException(info, "MailInfo");
			validation.validate(info, errors);
			
			if(errors.getErrorCount()>0){
				BindingResult rs = errors.getBindingResult();
				List<ObjectError> list = rs.getAllErrors();
				Iterator<ObjectError> iterator = list.iterator();
				String errorMsg = "";
				while(iterator.hasNext()){
					ObjectError error = (ObjectError)iterator.next();
					errorMsg += error.getDefaultMessage() + System.getProperty("line.separator");
				}
				JOptionPane.showMessageDialog(null, errorMsg, Constant.WARNING_MSG_WARNING, JOptionPane.WARNING_MESSAGE);
			}else{
				LittleOne l = new LittleOne();
				try {
					l.mailsend(info);
					updateData(queTable.getSelectedRow(), queTable, info.getReciver(), info.getUuid());
					this.dispose();
				} catch (AddressException e1) {
					e1.printStackTrace();
				} catch (MessagingException e1) {
					JOptionPane.showMessageDialog(null, Constant.WARRING_MSG_CHECK_ID_PASS, Constant.WARNING_MSG_WARNING, JOptionPane.WARNING_MESSAGE);
				}
			}
		}else if(cancelBtn.equals(btn)){
			this.dispose();
		}
	}
	
	private void setTextFieldsValue(){
		info.setContext(taContent.getText());
		info.setReciver(tfReciver.getText());
		info.setPassword(String.valueOf(tfPass.getPassword()));
		info.setSubject(tfTitle.getText());
	}
	
	private void updateData(int row,JTable queTable, String mail, String uuid){
		//存到資料庫、改變狀態
		//存到資料庫後檢查是否成功新增
		QuestionnairesTableModel qModel = (QuestionnairesTableModel) queTable.getModel();
		int i = saveDB(qModel.getValueAt(row, qModel.findColumn(Constant.ID)), uuid);
		String msg =  i>0 ? Constant.EXP_SURVEY_MSG_MAIL_SENT : Constant.EXP_SURVEY_MSG_MAIL_SEND_FAILED;
		JOptionPane.showMessageDialog(this, msg, msg, i>0 ? JOptionPane.INFORMATION_MESSAGE:JOptionPane.ERROR_MESSAGE);
		if(i <= 0){
			return;
		}
		//儲存失敗則不做下列動作
		Questionnaire q = ((QuestionnairesTableModel)queTable.getModel()).getExperiment().getQuestionnaires().get(row);
		q.setState(Questionnaire.State.SENT);
		q.setSubject(mail);
		//Fire Table
		((QuestionnairesTableModel)queTable.getModel()).fireTableRowsUpdated(row, row);
	}
	
	private int saveDB(Object questionnaireId, String uuid){
		return DAO.insert(String.format("insert into uuidmapping values (null,'%s','%s')", uuid, questionnaireId));
	}
}
