package main.phrasemodel.controller.ManualAdd;

import java.util.ArrayList;
import java.util.Iterator;

import javax.swing.JOptionPane;

import main.phrasemodel.management.model.ExperModel;
import main.phrasemodel.management.view.ExperPanel;
import main.phrasemodel.model.ManualAdd.ManualAddModel;
import main.phrasemodel.view.PhraseView;
import main.phrasemodel.view.ManualAdd.SelectState;
import main.phrasemodel.view.ManualAdd.State;
import main.phrasemodel.vo.ExperData;
import main.phrasemodel.vo.PhraseData;

import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.support.rowset.SqlRowSet;

import tools.db.DAO;
import tools.util.Constant;
import vo.CustomMap;
//XXX 跟資料庫有關的東西要抽到 model 去  演算法也要抽走
public class ManualAddMainController {
	private JdbcTemplate jdbcTemplate = DAO.getJdbcTemplate();
	private ManualAddModel model;
	private ArrayList<String> phrases;
	private ArrayList<String> replacePhrases;
	private ArrayList<String> group;
	private ArrayList<String> result;
	private ArrayList<CustomMap> splitGroup;
	//type = 0 =>  英文割字演算法, type = 1 =>  中文割字演算法, type = 2 =>  不使用割字演算法, type = 3 =>  手動演算法
	private int type;
	private State state;
	private State modifyState;
	private State selectState;
	private State cancelState;
	private State replaceState;
	private State finishModifyState;
	
	public ManualAddMainController(PhraseData pd, PhraseView phraseView, ExperPanel exper){
		model = new ManualAddModel(pd);
		model.registerModelObserver(exper);
		model.registerModelObserver(phraseView);
		selectState = new SelectState(this, model, pd);
		state = selectState;
		state.createView();
	}
	
	public boolean insertExperDataByDemoData(ExperData data) {
		ExperModel eModel = new ExperModel();
		if (eModel.isRowExist(data)) {
			JOptionPane.showMessageDialog(null, Constant.ERROR_MSG_CAN_NOT_INSERT, Constant.ERROR_MSG_ERROR, JOptionPane.ERROR_MESSAGE);
			return false;
		}
		if(data.isFieldDuplicate()){
			JOptionPane.showMessageDialog(null, Constant.ERROR_MSG_SENTENCE_CAN_NOT_REPEAT, Constant.ERROR_MSG_ERROR, JOptionPane.ERROR_MESSAGE);
			return false;
		}
		if (eModel.isTitleDuplicate(data)) {
			JOptionPane.showMessageDialog(null, Constant.ERROR_MSG_TITLE_CAN_NOT_REPEAT, Constant.ERROR_MSG_ERROR, JOptionPane.ERROR_MESSAGE);
			return false;
		}
		if("".equals(data.getTitle())){
			JOptionPane.showMessageDialog(null, Constant.WARNING_MSG_PLEASE_INSERT, Constant.WARNING_MSG_WARNING, JOptionPane.WARNING_MESSAGE);
			return false;
		}
		model.doInsert(data);
		return true;
	}
	
	public void createView(){
		state.createView();
	}

	public void next() {
		state.next();
	}

	public void prev() {
		state.prev();
	}

	public void cancel() {
		state.cancel();
	}
	
	public ArrayList<String> getPhrases() {
		return phrases;
	}
	
	public void setPhrases(ArrayList<String> phrases) {
		this.phrases = phrases;
	}
	
	public ArrayList<String> getReplacePhrases() {
		return replacePhrases;
	}
	
	public void setReplacePhrases(ArrayList<String> replacePhrases) {
		this.replacePhrases = replacePhrases;
	}
	
	public State getFinishModifyState() {
		return finishModifyState;
	}

	public void setFinishModifyState(State finishModifyState) {
		this.finishModifyState = finishModifyState;
	}

	public int getType() {
		return type;
	}
	
	public void setType(int type) {
		this.type = type;
	}
	
	public State getState() {
		return state;
	}
	
	public void setState(State state) {
		this.state = state;
	}
	
	public State getModifyState() {
		return modifyState;
	}

	public void setModifyState(State modifyState) {
		this.modifyState = modifyState;
	}

	public State getSelectState() {
		return selectState;
	}

	public void setSelectState(State selectState) {
		this.selectState = selectState;
	}

	public void setReplaceState(State replaceState) {
		this.replaceState = replaceState;
	}
	
	public State getCancelState() {
		return cancelState;
	}
	
	public void setCancelState(State cancelState) {
		this.cancelState = cancelState;
	}
	
	public State getReplaceState() {
		return replaceState;
	}

	public ArrayList<String> getGroup() {
		return group;
	}

	public ArrayList<CustomMap> getSplitGroup() {
		return splitGroup;
	}
	
	public ArrayList<String> getResult() {
		return result;
	}
	
	/**
	 * 呼叫 model 取得當前 table 的最大 id + 1 的結果。
	 * @return int Max Id
	 */
	public int autoGenerateTitle() {
		return model.getMaxId() + 1;
	}
	
	//XXX 有空再抽
	public void execSpiltAlgorithm(){
		group = new ArrayList<String>();
		Iterator<String> iterator = phrases.iterator();
		
		JdbcTemplate jdbcTemplate = DAO.getJdbcTemplate();
		jdbcTemplate.execute("delete from allwords;");
		jdbcTemplate.execute("delete from words;");
		jdbcTemplate.execute("delete from independentwords;");
		
		ArrayList<String> splitWordTemp = new ArrayList<String>();
		
		int j = 1;
		//Step 1.  割字
		while(iterator.hasNext()){
			String str = iterator.next().trim();
			//分割句子 and insert into db
			String[] words = null;
			
			splitWordTemp.clear();
			
			if(0 == type){
				words = str.split("\\s+");
				
				for(int i = 0; i < words.length; i++){
					//假如有包含標點符號
					if(words[i].matches("(.)*\\p{P}") || words[i].matches("\\p{P}(.)*")){
						//有包含標點符號就要逐字檢查
						String wt = "";
						for(int k = 0; k < words[i].length(); k++){
							if(words[i].substring(k, k+1).matches("\\p{P}")){
								if(!"".equals(wt)){
									splitWordTemp.add(wt);
									wt = "";
								}
								splitWordTemp.add(words[i].substring(k,k+1));
							}else{
								wt += words[i].substring(k, k+1);
							}
						}
						if(!"".equals(wt)){
							splitWordTemp.add(wt);
							wt = "";
						}
					}else{
						splitWordTemp.add(words[i]);
					}
				}
				
			}else if(1 == type){
				words = new String[str.length()];
				for(int i = 0; i < str.length(); i++){
					splitWordTemp.add(str.substring(i,i+1));
				}
			}else if(2 == type){
				splitWordTemp.add(str);
			}else if(3 == type){
				words = str.split("\\|");
				
				for(int i = 0; i < words.length; i++){
					//假如有包含標點符號
					String word = words[i].trim();
					if(word.matches("(.)*\\p{P}") || word.matches("\\p{P}(.)*")){
						//有包含標點符號就要逐字檢查
						String wt = "";
						for(int k = 0; k < word.length(); k++){
							if(word.substring(k, k+1).matches("\\p{P}")){
								if(!"".equals(wt)){
									splitWordTemp.add(wt);
									wt = "";
								}
								splitWordTemp.add(word.substring(k,k+1));
							}else{
								wt += word.substring(k, k+1);
							}
						}
						if(!"".equals(wt)){
							splitWordTemp.add(wt);
							wt = "";
						}
					}else{
						splitWordTemp.add(word);
					}
				}
			}
			
			for(int i = 0; i < splitWordTemp.size(); i++){
				//理論上一個句子應該不會擁有 1000 個以上的單字吧
				if(2 > type){
					jdbcTemplate.update("insert into allwords values(?, ?, ?);", (j * 1000 + i), splitWordTemp.get(i).toLowerCase().trim(), splitWordTemp.get(i).trim());
				}else{
					jdbcTemplate.update("insert into allwords values(?, ?, ?);", (j * 1000 + i), splitWordTemp.get(i).toLowerCase().trim(), splitWordTemp.get(i).trim() + "|");
				}
			}
			j++;
		}
		
		if(2 > type){
			step2();
		}else{
			SqlRowSet rs = jdbcTemplate.queryForRowSet("select id, word from allwords");
			
			while(rs.next()){
				if(!rs.getString("word").matches("\\p{P}") && !group.contains(rs.getString("word")) && !rs.getString("word").matches("\\s+") && !"".equals(rs.getString("word"))){
					group.add(rs.getString("word"));
				}
			}
			
			addSeparatorLine(jdbcTemplate, rs, phrases);
		}
	}
	
	//把有重複出現的字放到 words Table 中
	private void step2(){
		//XXX 可以改成 set 去做  利用不可重複的特性
		SqlRowSet rs = jdbcTemplate.queryForRowSet("select word, count(word) as wordnums, oword from allwords group by word");
		
		//當字的個數超過一個以上時就新增到 wors table 中，否則新增到 independentwords table 中。
		while(rs.next()){
			//不是標點符號
			if(!rs.getString("word").matches("\\p{P}|\\s+")){
				if(rs.getInt("wordnums")>1){
					jdbcTemplate.update("insert into words values(null, ?);", rs.getString("word"));
				}else{
					jdbcTemplate.update("insert into independentwords values(null, ?);", rs.getString("word"));
				}
			} else{
				if(!rs.getString("oword").endsWith("|")){
					jdbcTemplate.update("update allwords set oword = oword || ? where oword = ?;", "|", rs.getString("oword"));
				}
			}
		}
		
		//處理 words 中的字。
		ArrayList<String> word = getWordList(jdbcTemplate, "words");
		setWordGroup(jdbcTemplate, rs, word, group);
		
		//處理 independentwords 中的字。
		word = getWordList(jdbcTemplate, "independentwords");
		setIndependentWordsGroup(jdbcTemplate, rs, word, group);
		
		addSeparatorLine(jdbcTemplate, rs, phrases);
		
		jdbcTemplate.update("update allwords set word = ?, oword = trim(replace(oword,?,?));", "", "|", "");
		jdbcTemplate.execute("delete from words;");
		jdbcTemplate.execute("delete from independentwords;");
	}
	
	private ArrayList<String> getWordList(JdbcTemplate jdbcTemplate, String tableName){
		ArrayList<String> word = new ArrayList<String>();
		SqlRowSet rs = jdbcTemplate.queryForRowSet("select id, word from allwords where word in (select word from " + tableName + ")");
		jdbcTemplate.execute("delete from words;");
		
		while(rs.next()){
			jdbcTemplate.update("insert into words values(?, ?);", rs.getInt("id"), rs.getString("word"));
			word.add(rs.getString("word"));
		}
//		word.add("");
		return word;
	}
	
	private void setWordGroup(JdbcTemplate jdbcTemplate, SqlRowSet rs, ArrayList<String> word, ArrayList<String> group){
		String groupStr = "";
		String nowStr = "";
		String nextStr = "";
		int count = 0;
		SqlRowSet rs2;
		
		for(int i = 0; i < word.size();){
			
			if(count == 0){
				nowStr = word.get(i);
				nextStr = "";
				rs = jdbcTemplate.queryForRowSet("select id from words where word = ?", nowStr);
				while(rs.next()){					
					rs2 = jdbcTemplate.queryForRowSet("select id, word from words where id = ?", rs.getInt("id")+1);
					if(rs2.next()){
						if("".equals(nextStr)){
							nextStr = rs2.getString("word");
						}else{
							if(!nextStr.equals(rs2.getString("word"))){
								count++;
							}
						}
					}else{
						count++;
					}
				}
				
				rs = jdbcTemplate.queryForRowSet("select id from words where word = ?", nextStr);
				while(rs.next()){					
					rs2 = jdbcTemplate.queryForRowSet("select id, word from words where id = ?", rs.getInt("id")-1);
					if(rs2.next()){
						if(!nowStr.equals(rs2.getString("word"))){
							count++;
						}
					}else{
						count++;
					}
				}
				
				groupStr += nowStr;
				if(0 == type){
					groupStr += " ";
				}
				i++;
			}else{
				groupStr = groupStr.trim();
				if(!group.contains(groupStr) && !groupStr.matches("\\s+") && !"".equals(groupStr)){
					group.add(groupStr);
					
					//XXX 每次都判斷  很奇怪
					String sp = "";
					String line = "";
					if(0 == type){
						line = " |";
						if(groupStr.contains(" ")){
							String[] tmp = groupStr.split("\\s");
							sp = tmp[tmp.length-1];
						}else{
							sp = groupStr;
						}
					}else if(1 == type){
						line = "|";
						if(groupStr.length() > 0){
							sp = groupStr.substring(groupStr.length() - 1);
						}
					}
					int num = jdbcTemplate.queryForInt("select count(word) as num from allwords where word = ?", sp);
					if(num > 0){
						jdbcTemplate.update("update allwords set oword = oword || ? where word = ?", line, sp);
					}
				}
				groupStr = "";
				count = 0;
				nextStr ="";
				nowStr = "";
			}
		}
	}
	
	private void setIndependentWordsGroup(JdbcTemplate jdbcTemplate, SqlRowSet rs, ArrayList<String> word, ArrayList<String> group){
		for(String s : word){
			if(!group.contains(s) && !s.matches("\\s+") && !"".equals(s)){
				group.add(s);
				String line = "";
				if(0 == type){
					line = " |";
				}else if(1 == type){
					line = "|";
				}
				int num = jdbcTemplate.queryForInt("select count(word) as num from allwords where word = ?", s);
				if(num > 0){
					jdbcTemplate.update("update allwords set oword = oword || ? where word = ?", line, s);
				}
			}
		}
	}
	
	private void addSeparatorLine(JdbcTemplate jdbcTemplate, SqlRowSet rs, ArrayList<String> phrases){
		result = new ArrayList<String>();
		splitGroup = new ArrayList<CustomMap>();
		
		if(3 == type){
			for(int i = 0; i < phrases.size(); i++){
				if(phrases.get(i).matches("[A-Za-z]+(.)*")){
					phrases.set(i, phrases.get(i).replace("|", " "));
					phrases.set(i, phrases.get(i).replaceAll("\\s+", " "));
					phrases.set(i, phrases.get(i).trim());
				}else{
					phrases.set(i, phrases.get(i).replace("|", ""));
					phrases.set(i, phrases.get(i).trim());
				}
			}
		}
		
		for(int i = 1000; i <= phrases.size() * 1000; i+= 1000){
			String str = "";
			rs = jdbcTemplate.queryForRowSet("select oword from allwords where id between ? and ? order by id asc", i, i + 1000 -1);
			while(rs.next()){
				if(rs.getString("oword").matches("[A-Za-z]+(.)*")){
					str += " ";
				}
				str += rs.getString("oword");
			}
			if(str.trim().endsWith("|")){
				str = str.substring(0, str.length() - 1);
			}
			str = str.trim();
			result.add(str);
			CustomMap aPhrase = new CustomMap();
			String[] words = str.split("\\|");
			for(String word: words){
				aPhrase.putMapListValue(word.trim(), word.trim());
			}
			splitGroup.add(aPhrase);
		}
	}
}
