package main.phrasemodel.management.controller;

import javax.swing.JOptionPane;

import main.phrasemodel.management.model.EDModel;
import main.phrasemodel.management.view.ManagementNewDialog;
import main.phrasemodel.validation.EDValidator;
import main.phrasemodel.vo.ED;
import main.phrasemodel.vo.PhraseData;

import org.springframework.validation.BindException;

import tools.util.Constant;

public class ManagementNewController {
	
	private EDModel model;
	private ManagementNewDialog dialog;
	
	public ManagementNewController(PhraseData pd, EDModel model, String title){
		this.model = model;
		
		dialog = new ManagementNewDialog(pd, this, model);
		dialog.createView(title);
	}

	public void addEDPhrase(int factorCount) {
		ED demoData = dialog.getInsertData();
		EDValidator validator = new EDValidator(factorCount);
		BindException errors = new BindException(demoData, "DemoData");
		validator.validate(demoData, errors);
		
		if (errors.getErrorCount()>0) {
			JOptionPane.showMessageDialog(null, Constant.WARNING_MSG_MUST_HAVE_VALUE, Constant.WARNING_MSG_WARNING, JOptionPane.WARNING_MESSAGE);
			return;
		}
		
		if(demoData.isFieldDuplicate()){
			JOptionPane.showMessageDialog(null, Constant.ERROR_MSG_SENTENCE_CAN_NOT_REPEAT, Constant.ERROR_MSG_ERROR, JOptionPane.ERROR_MESSAGE);
			return;
		}
		
		if (model.isRowExist(demoData)) {
			JOptionPane.showMessageDialog(null, Constant.ERROR_MSG_CAN_NOT_INSERT, Constant.ERROR_MSG_ERROR, JOptionPane.ERROR_MESSAGE);
			return;
		}
		
		if (model.isTitleDuplicate(demoData)) {
			JOptionPane.showMessageDialog(null, Constant.ERROR_MSG_TITLE_CAN_NOT_REPEAT, Constant.ERROR_MSG_ERROR, JOptionPane.ERROR_MESSAGE);
			return;
		}
		
		model.doInsert(demoData);
		dialog.dispose();
	}
	
	/**
	 * 呼叫 model 取得當前 table 的最大 id + 1 的結果。
	 * @return int Max Id
	 */
	public int autoGenerateTitle() {
		return model.getMaxId();
	}
}
