package main.phrasemodel.management.model;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.swing.JTable;

import main.phrasemodel.observer.DemoObserver;
import main.phrasemodel.observer.DemoSubject;
import main.phrasemodel.observer.ModelObserver;
import main.phrasemodel.observer.ModelSubject;
import main.phrasemodel.vo.ED;
import main.phrasemodel.vo.PhraseData;

import org.springframework.jdbc.core.BatchPreparedStatementSetter;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.PreparedStatementCreator;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.jdbc.support.GeneratedKeyHolder;
import org.springframework.jdbc.support.KeyHolder;

import tools.db.DAO;
import tools.util.Constant;

public abstract class EDModel implements ModelSubject, DemoSubject {
	private ArrayList<ModelObserver> subject = new ArrayList<ModelObserver>();
	private ArrayList<DemoObserver> demoSubject = new ArrayList<DemoObserver>();
	private static JdbcTemplate jdbcTemplate = DAO.getJdbcTemplate();
	private String tableName;
	
	public EDModel(String tableName){
		this.tableName = tableName;
	}
	
	//XXX 似乎不該出現在此。
	public SuperTableModel createTableModel(PhraseData pd) {
		String[] columnNames = null;
		String factor1 = pd.getFactor1();
		String factor2 = pd.getFactor2();
		if(pd.getFactorCount() == 2){
			columnNames = new String[]{
					Constant.NUMOF,
					Constant.ID,
					Constant.FACTOR_MANAGE_TITLE,
					"+" + factor1.substring(0,1) + ", +" + factor2.substring(0,1),
					"+" + factor1.substring(0,1) + ", -" + factor2.substring(0,1),
					"-" + factor1.substring(0,1) + ", +" + factor2.substring(0,1),
					"-" + factor1.substring(0,1) + ", -" + factor2.substring(0,1)
			};
		}else{
			columnNames = new String[]{
					Constant.NUMOF,
					Constant.ID,
					Constant.FACTOR_MANAGE_TITLE,
					"+" + factor1.substring(0,1),
					"-" + factor1.substring(0,1),
			};
		}
		
		String sql = "select * from " + tableName + " where phrase_id = ? order by id desc;";
		
		List<ArrayList<Object[]>> dataList = jdbcTemplate.query(sql, new Object[]{pd.getId()}, new RowMapper<ArrayList<Object[]>>(){
			public ArrayList<Object[]> mapRow(ResultSet rs, int i)
					throws SQLException {
				ArrayList<Object[]> list = new ArrayList<Object[]>();
				int j = 0;
				do{
					j++;
					ED data = new ED(rs);
					list.add(data.toObjectArray(j));
				}while(rs.next());
				return list;
			}
		});
		
		ArrayList<Object[]> data;
		if(dataList.size() == 0){
			data = new ArrayList<Object[]>();
		}else{
			data = dataList.get(0);
		}
		
		//XXX Extract
		if(data.size() > 0){
			notifyDemoObserver(true);
		}else{
			notifyDemoObserver(false);
		}
		
		return new SuperTableModel(columnNames, data);
	}
	
	public ED select(String id) {
		String sql = "select * from " + tableName + " where id = ?;";
		ED data = jdbcTemplate.queryForObject(sql, new Object[]{id}, new RowMapper<ED>(){
			public ED mapRow(ResultSet rs, int i)
					throws SQLException {
				return new ED(rs);
			}
		});
		return data;
	}
	
	public ArrayList<ED> getPhraseAllExpers(String id) {
		final ArrayList<ED> list = new ArrayList<ED>();
		String sql = "select * from " + tableName + " where phrase_id = ? order by id;";
		jdbcTemplate.queryForObject(sql, new Object[]{id}, new RowMapper<ArrayList<ED>>(){
			public ArrayList<ED> mapRow(ResultSet rs, int i)
					throws SQLException {
				do{
					list.add(new ED(rs));
				}while(rs.next());
				
				return list;
			}
		});
		return list;
	}
	
	/**
	 * 取得當前 table 中最大的 id。
	 * @return int Max Id
	 */
	public int getMaxId() {
		String sql = "select MAX(id) from " + tableName + ";";
		return jdbcTemplate.queryForInt(sql) + 1;
	}
	
	//====================insert====================================================
	
	public void doInsert(ED data) {
		final String sql = "insert into " + tableName + " values(null, ?, ?, ?, ?, ?, ?);";
		insert(sql, data);
		notifyDoInsert(data);
		notifyModelObserver();
	}
	
	protected Number insert(final String sql, final ED data){
		KeyHolder keyHolder = new GeneratedKeyHolder();
		jdbcTemplate.update(new PreparedStatementCreator(){
			public PreparedStatement createPreparedStatement(Connection conn)
					throws SQLException {
				//XXX 抽出去。
				PreparedStatement ps = conn.prepareStatement(sql);
				ps.setObject(1, data.getTitle());
				ps.setObject(2, data.getCase1());
				ps.setObject(3, data.getCase2());
				ps.setObject(4, data.getCase3());
				ps.setObject(5, data.getCase4());
				ps.setObject(6, data.getPhraseId());
	            return ps;
			}}, keyHolder);
		return keyHolder.getKey();
	}
	
	public boolean isRowExist(ED demoData) {
		String sql = "";
		Object[] object = null;
		
		int num = 0;
		for(int i = 0; i < demoData.getFactorCount()*2; i++){
			if(demoData.getFactorCount() == 1){
				sql = "select count(*) from " + tableName + " where case" + (i+1) + " in (?, ?) and case3 is null and case4 is null and id <> ? and phrase_id = ?;";
				object = new Object[]{demoData.getCase1(), demoData.getCase2(), demoData.getId(), demoData.getPhraseId()};
			}else{
				sql = "select count(*) from " + tableName + " where case" + (i+1) + " in (?, ?, ? ,?) and id <> ? and phrase_id = ?;";
				object = new Object[]{demoData.getCase1(), demoData.getCase2(), demoData.getCase3(), demoData.getCase4(), demoData.getId(), demoData.getPhraseId()};
			}
			num += jdbcTemplate.queryForInt(sql, object);
		}
		
		if(num>0){
			return true;
		}else{
			return false;
		}
	}
	
	public boolean isTitleDuplicate(ED demoData) {
		String sql = "";
		Object[] object = null;
		sql = "select count(*) from " + tableName + " where title = ? and id <> ? and phrase_id = ?;";
		object = new Object[]{demoData.getTitle(), demoData.getId(), demoData.getPhraseId()};
		
		int num = jdbcTemplate.queryForInt(sql, object);
		
		if(num>0){
			return true;
		}else{
			return false;
		}
	}
	
	//====================delete====================================================
	
	public void doDelete(JTable table) {
		int[] rowIds = table.getSelectedRows();
		String[] ids  = selectedId(rowIds, table);
		//XXX 刪除失敗要怎處理？
		delete(ids);
		notifyModelObserver();
	}
	
	//XXX Extract
	protected String[] selectedId(int[] selectedRowId, JTable table){
		String[] ids = new String[selectedRowId.length];
		SuperTableModel stModel = (SuperTableModel) table.getModel();
		for(int i = 0; i < selectedRowId.length; i++){
			ids[i] = String.valueOf(stModel.getValueAt(selectedRowId[i], stModel.findColumn(Constant.ID)).toString());
		}
		return ids;
	}
	
	protected int[] delete(final Object[] ids){
		String sql = "delete from " + tableName + " where id = ?;";
	    BatchPreparedStatementSetter setter = new BatchPreparedStatementSetter(){
	        public void setValues(
	            PreparedStatement ps, int i) throws SQLException {
	            ps.setObject(1, ids[i], Types.INTEGER);
	        }
	        
			public int getBatchSize() {
				return ids.length;
			}
		};
		return jdbcTemplate.batchUpdate(sql, setter);
	}
	
	//====================update====================================================
	
	public void doUpdate(ED data) {
		String sql = "update " + tableName + " set title = ?, case1 = ?, case2 = ?, case3 = ?, case4 = ? where id = ?;";
		update(sql, data);
		notifyModelObserver();
	}
	
	//XXX 可能可以跟 insert 合併。
	protected Number update(final String sql, final ED data){
		KeyHolder keyHolder = new GeneratedKeyHolder();
		jdbcTemplate.update(new PreparedStatementCreator(){
			public PreparedStatement createPreparedStatement(Connection conn)
					throws SQLException {
				//XXX 抽出去。
				PreparedStatement ps = conn.prepareStatement(sql);
				ps.setObject(1, data.getTitle());
				ps.setObject(2, data.getCase1());
				ps.setObject(3, data.getCase2());
				ps.setObject(4, data.getCase3());
				ps.setObject(5, data.getCase4());
				ps.setObject(6, data.getId());
	            return ps;
			}}, keyHolder);
		return keyHolder.getKey();
	}
	
	//====================delete====================================================
	
	public void notifyModelObserver() {
		Iterator<ModelObserver> iterator = subject.iterator();
		while(iterator.hasNext()){
			ModelObserver mo = iterator.next();
			mo.refresh();
		}
	}
	
	public void registerModelObserver(ModelObserver o) {
		subject.add(o);
	}
	
	public void removeModelObserver(ModelObserver o) {
		subject.remove(o);
	}
	
	public void notifyDemoObserver(boolean enable) {
		Iterator<DemoObserver> iterator = demoSubject.iterator();
		while(iterator.hasNext()){
			DemoObserver mo = iterator.next();
			mo.setEnable(enable);
		}
	}
	
	public void notifyDoInsert(ED data) {
		Iterator<DemoObserver> iterator = demoSubject.iterator();
		while(iterator.hasNext()){
			DemoObserver mo = iterator.next();
			mo.doInsert(data);
		}
	}
	
	public void registerDemoObserver(DemoObserver o) {
		demoSubject.add(o);
	}
	
	public void removeDemoObserver(DemoObserver o) {
		demoSubject.remove(o);		
	}
}