package main.phrasemodel.management.view;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import org.springframework.validation.BindException;

import main.phrasemodel.management.controller.ManagementController;
import main.phrasemodel.management.model.EDModel;
import main.phrasemodel.management.model.SuperTableModel;
import main.phrasemodel.observer.DemoObserver;
import main.phrasemodel.observer.ModelObserver;
import main.phrasemodel.validation.EDValidator;
import main.phrasemodel.vi.ToolTipTextTable;
import main.phrasemodel.vo.ED;
import tools.util.Constant;

public abstract class EDPanel extends JPanel implements ActionListener, ModelObserver, DemoObserver {
	private static final long serialVersionUID = 390847530249395119L;
	private EDModel model;
	private ManagementController controller;
	private SuperTableModel tableModel;
	private JButton newBtn;
	private JButton delBtn;
	private JButton autoBtn;
	private ToolTipTextTable table;
	private ManagementView view;
	private String title;
	
	public EDPanel(ManagementView view, String title, EDModel model){
		this.view = view;
		this.title = title;
		
		controller = view.getController();
		this.model = model;
		model.registerModelObserver(this);
		createView(title);
	}
	
	public void createView(String title){
		setBorder(BorderFactory.createTitledBorder(title));
		setLayout(new BorderLayout());
		
		JPanel btnPanel = new JPanel();
		
		newBtn = new JButton(Constant.FACTOR_MANAGE_DEMO_INSERT);
		delBtn = new JButton(Constant.FACTOR_MANAGE_DELETE);
		
		newBtn.addActionListener(this);
		delBtn.addActionListener(this);
		
		btnPanel.add(newBtn);
		if(isExperModel()){
			autoBtn = new JButton(Constant.FACTOR_MANAGE_EXPER_FROMDEMO);
			autoBtn.addActionListener(this);
			btnPanel.add(autoBtn);
		}
		btnPanel.add(delBtn);
		
		add(btnPanel, BorderLayout.NORTH);
		add(createTable(), BorderLayout.CENTER);
	}
	
	protected JScrollPane createTable() {
		table = new ToolTipTextTable();
		
		table.addMouseListener(new MouseAdapter(){
			public void mouseClicked(MouseEvent e){
				if(isDoubleClickJTable(e)){
					controller.showEditDialg(view.getPd(), table, model, title);
				}
			}
		});
		
		table.setPreferredScrollableViewportSize(new Dimension(500, 100));
		table.setFillsViewportHeight(true);
		refresh();
		return new JScrollPane(table);
	}
	
	protected boolean isDoubleClickJTable(MouseEvent e){
		return e.getClickCount() == 2 && isClickOnTableRow(e);
	}
	
	protected boolean isClickOnTableRow(MouseEvent e){
		Point p = e.getPoint();
		return table.rowAtPoint(p) > -1;
	}
	
	protected void setTableModel() {
		tableModel = model.createTableModel(view.getPd());
		table.setModel(tableModel);
	}
	
	public abstract boolean isExperModel();
	
	public void actionPerformed(ActionEvent e) {
		
		Object btn = e.getSource();
		if(btn.equals(newBtn)){
			controller.showNewDialg(view.getPd(), model, title);
		}else if(btn.equals(delBtn)){
			//XXX Extract?
			if(table.getSelectedRowCount() == 0 ){
				JOptionPane.showMessageDialog(null, Constant.ERROR_MSG_NOT_SELECT, Constant.ERROR_MSG_ERROR, JOptionPane.ERROR_MESSAGE);
			}else{
				int choose = JOptionPane.showConfirmDialog(null, Constant.MSG_IS_DELETE, Constant.MSG_NOTE, JOptionPane.YES_NO_OPTION);
				if( choose == JOptionPane.YES_OPTION ){
					controller.deletePhraseData(table, model);
				}
			}
		}else if(btn.equals(autoBtn)){
			controller.showManualAddDialog(view.getPd());
		}
	}
	
	public void registerDemoObserver(DemoObserver o){
		model.registerDemoObserver(o);
	}
	
	public void checkExperBtnState(DemoObserver o){
		if(table.getRowCount() == 0){
			model.notifyDemoObserver(false);
		}
	}
	
	public void doInsert(ED data){
		
		EDValidator validator = new EDValidator(data.getFactorCount());
		BindException errors = new BindException(data, "ExperData");
		validator.validate(data, errors);
		
		if (errors.getErrorCount()>0) {
			JOptionPane.showMessageDialog(null, Constant.WARNING_MSG_MUST_HAVE_VALUE, Constant.WARNING_MSG_WARNING, JOptionPane.WARNING_MESSAGE);
			return;
		}
		
		if(data.isFieldDuplicate()){
			JOptionPane.showMessageDialog(null, Constant.ERROR_MSG_SENTENCE_CAN_NOT_REPEAT, Constant.ERROR_MSG_ERROR, JOptionPane.ERROR_MESSAGE);
			return;
		}
		
		if (model.isRowExist(data)) {
			JOptionPane.showMessageDialog(null, Constant.ERROR_MSG_CAN_NOT_INSERT, Constant.ERROR_MSG_ERROR, JOptionPane.ERROR_MESSAGE);
			return;
		}
		
		if (model.isTitleDuplicate(data)) {
			JOptionPane.showMessageDialog(null, Constant.ERROR_MSG_TITLE_CAN_NOT_REPEAT, Constant.ERROR_MSG_ERROR, JOptionPane.ERROR_MESSAGE);
			return;
		}
		
		controller.doInsert(data, model);
	}
	
	public void setEnable(boolean enable) {
		autoBtn.setEnabled(enable);
	}
	
	public void refresh() {
		setTableModel();
		fireTableModelChanged();
		table.removeColumn(table.getColumn(Constant.ID));
		table.removeColumn(table.getColumn(Constant.NUMOF));
	}
	
	protected void fireTableModelChanged(){
		tableModel.fireTableDataChanged();
		tableModel.fireTableChanged(null);
	}
}
