package main.phrasemodel.model.ManualAdd;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import main.phrasemodel.management.model.EDModel;
import main.phrasemodel.management.model.ExperModel;
import main.phrasemodel.observer.ModelObserver;
import main.phrasemodel.observer.ModelSubject;
import main.phrasemodel.vo.DemoData;
import main.phrasemodel.vo.ExperData;
import main.phrasemodel.vo.PhraseData;

import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.PreparedStatementCreator;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.jdbc.support.GeneratedKeyHolder;
import org.springframework.jdbc.support.KeyHolder;

import tools.db.DAO;
import tools.util.Constant;

public class ManualAddModel implements ModelSubject {
	private static JdbcTemplate jdbcTemplate = DAO.getJdbcTemplate();
	private PhraseData pd;
	private ArrayList<ModelObserver> subject = new ArrayList<ModelObserver>();
	
	public ManualAddModel(PhraseData pd){
		this.pd = pd;
	}
	
	public ArrayList<DemoData> getDemoDataList(){
		String sql = "select * from " + Constant.DEMONSTRATION_TABLE + " where phrase_id = ? order by id desc;";
		ArrayList<DemoData> list = new ArrayList<DemoData>();
		List<ArrayList<DemoData>> dataList = jdbcTemplate.query(sql, new Object[]{pd.getId()}, new RowMapper<ArrayList<DemoData>>(){
			public ArrayList<DemoData> mapRow(ResultSet rs, int i)
					throws SQLException {
				ArrayList<DemoData> list = new ArrayList<DemoData>();
				do{
					DemoData demo = new DemoData(rs);
					list.add(demo);
				}while(rs.next());
				return list;
			}
		});
		
		if(dataList.size() > 0){
			list = dataList.get(0);
		}
		
		return list;
	}
	
	/**
	 * 取得當前 table 中最大的 id。
	 * @return int Max Id
	 */
	public int getMaxId() {
		EDModel model = new ExperModel();
		return model.getMaxId();
	}
	
	public void doInsert(ExperData data){
		final String sql = "insert into " + Constant.EXPERIMENTAL_TABLE + " values(null, ?, ?, ?, ?, ?, ?);";
		insert(sql, data);
		notifyModelObserver();
	}
	
	protected Number insert(final String sql, final ExperData data){
		KeyHolder keyHolder = new GeneratedKeyHolder();
		jdbcTemplate.update(new PreparedStatementCreator(){
			public PreparedStatement createPreparedStatement(Connection conn)
					throws SQLException {
				//XXX 抽出去。
				PreparedStatement ps = conn.prepareStatement(sql);
				ps.setObject(1, data.getTitle());
				ps.setObject(2, data.getCase1());
				ps.setObject(3, data.getCase2());
				ps.setObject(4, data.getCase3());
				ps.setObject(5, data.getCase4());
				ps.setObject(6, data.getPhraseId());
	            return ps;
			}}, keyHolder);
		return keyHolder.getKey();
	}
	
	public void notifyModelObserver() {
		Iterator<ModelObserver> iterator = subject.iterator();
		while(iterator.hasNext()){
			ModelObserver mo = iterator.next();
			mo.refresh();
		}
	}
	
	public void registerModelObserver(ModelObserver o) {
		subject.add(o);
	}
	
	public void removeModelObserver(ModelObserver o) {
		subject.remove(o);
	}
}
