package main.phrasemodel.model;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.swing.JTable;

import main.phrasemodel.observer.ModelObserver;
import main.phrasemodel.observer.ModelSubject;
import main.phrasemodel.vo.PhraseData;

import org.springframework.jdbc.core.BatchPreparedStatementSetter;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.PreparedStatementCreator;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.jdbc.support.GeneratedKeyHolder;
import org.springframework.jdbc.support.KeyHolder;

import tools.db.DAO;
import tools.util.Constant;

public class PhraseModel implements ModelSubject {
	private ArrayList<ModelObserver> subject = new ArrayList<ModelObserver>();
	private static JdbcTemplate jdbcTemplate = DAO.getJdbcTemplate();
	
	public PhraseModel(){
	}
	
	//XXX 做的事情似乎太多，可能要拆。
	//XXX 回傳型態?
	public PhraseTableModel createTableModel() {
		String[] columnNames = { 
				Constant.NUMOF,
				Constant.ID,
				Constant.FACTOR_FACTOR1,
				Constant.FACTOR_FACTOR2,
				Constant.FACTOR_DESCRIPTION,
				Constant.FACTOR_TOTAL_SENTENCES };
		
		String sql = "select *, ( select count(*) from " + 
			Constant.EXPERIMENTAL_TABLE + " where phrase_id = " + 
			Constant.PHRASE_TABLE + ".id ) as total_sentences from " + 
			Constant.PHRASE_TABLE + " order by id desc;";
		//FIXME 寫法很鳥。
		List<ArrayList<Object[]>> dataList = jdbcTemplate.query(sql, new RowMapper<ArrayList<Object[]>>(){
			public ArrayList<Object[]> mapRow(ResultSet rs, int i)
					throws SQLException {
				ArrayList<Object[]> list = new ArrayList<Object[]>();
				int j = 0;
				do{
					j++;
					PhraseData pd = new PhraseData(rs);
					list.add(pd.toObjectArray(j));
				}while(rs.next());
				return list;
			}
		});

		ArrayList<Object[]> data;
		if(dataList.size() == 0){
			data = new ArrayList<Object[]>();
		}else{
			data = dataList.get(0);
		}
		
		return new PhraseTableModel(columnNames, data);
	}
	
	public PhraseData select(String id) {
		String sql = "select *, ( select count(*) from " + 
		Constant.EXPERIMENTAL_TABLE + " where phrase_id = " + 
		Constant.PHRASE_TABLE + ".id ) as total_sentences from " + 
		Constant.PHRASE_TABLE + " where id = ?;";
		
		PhraseData phraseData = jdbcTemplate.queryForObject(sql, new Object[]{id}, new RowMapper<PhraseData>(){
			public PhraseData mapRow(ResultSet rs, int i)
					throws SQLException {
				PhraseData pd = new PhraseData(rs);
				return pd;
			}
		});
		return phraseData;
	}
	
	/**
	 * 取得句庫裡所有的資料。
	 * @return ArrayList<PhraseData>
	 */
	public static ArrayList<PhraseData> getAllPhrase() {
		String sql = "select *, ( select count(*) from " + 
		Constant.EXPERIMENTAL_TABLE + " where phrase_id = " + 
		Constant.PHRASE_TABLE + ".id ) as total_sentences from " + 
		Constant.PHRASE_TABLE + ";";
		
		ArrayList<PhraseData> list = jdbcTemplate.queryForObject(sql, new RowMapper<ArrayList<PhraseData>>() {
			public ArrayList<PhraseData> mapRow(ResultSet rs, int i)
					throws SQLException {
				ArrayList<PhraseData> list = new ArrayList<PhraseData>();
				do{
					list.add(new PhraseData(rs));
				}while(rs.next());
				return list;
			}
		});
		return list;
	}
	
	public boolean isRowExist(PhraseData pd){
		String sql = "";
		Object[] object = null;
		if(pd.getFactorCount() == 1){
			sql = "select count(*) from " + Constant.PHRASE_TABLE + " where factor1 = ? and factor2 is null and id <> ?;";
			object = new Object[]{pd.getFactor1(), pd.getId()};
		}else{
			sql = "select count(*) from " + Constant.PHRASE_TABLE + " where factor1 = ? and factor2 = ? and id <> ?;";
			object = new Object[]{pd.getFactor1(), pd.getFactor2(), pd.getId()};
		}
		
		int num = jdbcTemplate.queryForInt(sql, object);
		
		if(num>0){
			return true;
		}else{
			return false;
		}
	}
	
	//=============================insert======================================================
	
	public void doInsert(PhraseData pd){
		final String sql = "insert into " + Constant.PHRASE_TABLE + " values(null, ?, ?, ?);";
		insert(sql, pd);
		notifyModelObserver();
	}
	
	protected Number insert(final String sql, final PhraseData pd){
		KeyHolder keyHolder = new GeneratedKeyHolder();
		jdbcTemplate.update(new PreparedStatementCreator(){
			public PreparedStatement createPreparedStatement(Connection conn)
					throws SQLException {
				//XXX 抽出去。
				PreparedStatement ps = conn.prepareStatement(sql);
				ps.setObject(1, pd.getFactor1());
				if("".equals(pd.getFactor2())){
					ps.setObject(2, null);
				}else{
					ps.setObject(2, pd.getFactor2());
				}
				ps.setObject(3, pd.getDescription());
	            return ps;
			}}, keyHolder);
		return keyHolder.getKey();
	}
	
	//=================================delete=================================================
	
	public void doDelete(JTable table){
		//TODO 未選擇時要警告。
		int[] rowIds = table.getSelectedRows();
		String[] ids  = selectedPhraseId(rowIds, table);
		//XXX 刪除失敗要怎處理？
		//XXX 改成 trigger?
		delete(Constant.PHRASE_TABLE, ids);
		delete(Constant.DEMONSTRATION_TABLE, ids);
		delete(Constant.EXPERIMENTAL_TABLE, ids);
		notifyModelObserver();
	}
	
	protected String[] selectedPhraseId(int[] selectedRowId, JTable table){
		String[] ids = new String[selectedRowId.length];
		PhraseTableModel pModel = (PhraseTableModel) table.getModel();
		for(int i = 0; i < selectedRowId.length; i++){
			ids[i] = String.valueOf(pModel.getValueAt(selectedRowId[i], pModel.findColumn(Constant.ID)));
		}
		return ids;
	}
	
	protected int[] delete(String tableName, final Object[] ids){
		String sql = "";
		if(tableName.equals(Constant.PHRASE_TABLE)){
			sql = "delete from " + tableName + " where id = ?;";
		}else{
			sql = "delete from " + Constant.EXPERIMENTAL_TABLE + " where phrase_id = ?;";
		}
	    BatchPreparedStatementSetter setter = new BatchPreparedStatementSetter(){
	        public void setValues(
	            PreparedStatement ps, int i) throws SQLException {
	            ps.setObject(1, ids[i], Types.INTEGER);
	        }
	        
			public int getBatchSize() {
				return ids.length;
			}
		};
		return jdbcTemplate.batchUpdate(sql, setter);
	}
	
	//=============================Update======================================================
	
	
	public void doUpdate(PhraseData pd) {
		String sql = "update " + Constant.PHRASE_TABLE + " set factor1 = ?, factor2 = ?, description = ? where id = ?;";
		update(sql, pd);
		notifyModelObserver();
	}
	
	//XXX 可能可以跟 insert 合併。
	protected Number update(final String sql, final PhraseData pd){
		KeyHolder keyHolder = new GeneratedKeyHolder();
		jdbcTemplate.update(new PreparedStatementCreator(){
			public PreparedStatement createPreparedStatement(Connection conn)
					throws SQLException {
				//XXX 抽出去。
				PreparedStatement ps = conn.prepareStatement(sql);
				ps.setObject(1, pd.getFactor1());
				ps.setObject(2, pd.getFactor2());
				ps.setObject(3, pd.getDescription());
				ps.setObject(4, pd.getId());
	            return ps;
			}}, keyHolder);
		return keyHolder.getKey();
	}
	
	//=========================================================================================
	
	public void notifyModelObserver() {
		Iterator<ModelObserver> iterator = subject.iterator();
		while(iterator.hasNext()){
			ModelObserver mo = iterator.next();
			mo.refresh();
		}
	}
	
	public void registerModelObserver(ModelObserver o) {
		subject.add(o);
	}
	
	public void removeModelObserver(ModelObserver o) {
		subject.remove(o);
	}
}
