package main.phrasemodel.view;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.PrintWriter;

import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;

import main.phrasemodel.controller.PhraseMainController;
import main.phrasemodel.model.PhraseModel;
import main.phrasemodel.model.PhraseTableModel;
import main.phrasemodel.observer.ModelObserver;
import tools.util.Constant;

public class PhraseView extends JPanel implements ActionListener, ModelObserver {
	/**
	 * 
	 */
	private static final long serialVersionUID = 7554092323439234858L;
	private PhraseMainController controller;
	private PhraseModel model;
	private PhraseTableModel ptModel;
	private JButton newBtn;
	private JButton delBtn;
	private JButton editBtn;
	private JButton exportBtn;
	private JTable phraseTable;
	
	public PhraseView(PhraseMainController controller, PhraseModel model){
		this.controller = controller;
		this.model = model;
		model.registerModelObserver(this);
	}
	
	public void createView(){
		newBtn = new JButton(Constant.FACTOR_INSERT);
		delBtn = new JButton(Constant.FACTOR_DELETE);
		editBtn = new JButton(Constant.FACTOR_EDIT);
		exportBtn = new JButton(Constant.EXPORT);
		
		newBtn.addActionListener(this);
		delBtn.addActionListener(this);
		editBtn.addActionListener(this);
		exportBtn.addActionListener(this);
		
		setLayout(new BorderLayout());
		JPanel btnPanel = new JPanel();
		btnPanel.add(newBtn);
		btnPanel.add(delBtn);
		btnPanel.add(editBtn);
		btnPanel.add(exportBtn);
		add(btnPanel, BorderLayout.NORTH);
		add(createTable(), BorderLayout.CENTER);
	}
	
	protected JScrollPane createTable() {
		phraseTable = new JTable();
		phraseTable.addMouseListener(new MouseAdapter(){
			public void mouseClicked(MouseEvent e){
				if(isDoubleClickJTable(e)){
					controller.showManagementDialg(phraseTable);
				}
			}
		});
		phraseTable.setPreferredScrollableViewportSize(new Dimension(500, 100));
		phraseTable.setFillsViewportHeight(true);
		refresh();
		return new JScrollPane(phraseTable);
	}
	
	protected boolean isDoubleClickJTable(MouseEvent e){
		return e.getClickCount() == 2 && isClickOnTableRow(e);
	}
	
	protected boolean isClickOnTableRow(MouseEvent e){
		Point p = e.getPoint();
		return phraseTable.rowAtPoint(p) > -1;
	}
	
	protected void setTableModel() {
		ptModel = model.createTableModel();
		phraseTable.setModel(ptModel);
	}
	
	public void actionPerformed(ActionEvent e) {
		Object btn = e.getSource();
		if(btn.equals(newBtn)){
			controller.showNewDialg();
		}else if(btn.equals(delBtn)){
			//XXX Extract?
			if(phraseTable.getSelectedRowCount() == 0 ){
				JOptionPane.showMessageDialog(null, Constant.ERROR_MSG_NOT_SELECT, Constant.ERROR_MSG_ERROR, JOptionPane.ERROR_MESSAGE);
			}else{
				int choose = JOptionPane.showConfirmDialog(null, Constant.MSG_IS_DELETE, Constant.MSG_NOTE, JOptionPane.YES_NO_OPTION);
				if( choose == JOptionPane.YES_OPTION ){
					controller.deletePhraseData(phraseTable);
				}
			}
		}else if(btn.equals(editBtn)){
			if(phraseTable.getSelectedRowCount() == 0 ){
				JOptionPane.showMessageDialog(null, Constant.ERROR_MSG_NOT_SELECT, Constant.ERROR_MSG_ERROR, JOptionPane.ERROR_MESSAGE);
			}else{
				controller.showEditDialg(phraseTable);
			}
		}else if(btn.equals(exportBtn)){
			
			if(phraseTable.getSelectedRowCount() == 0){
				JOptionPane.showMessageDialog(null, Constant.ERROR_MSG_NOT_SELECT, Constant.ERROR_MSG_ERROR, JOptionPane.ERROR_MESSAGE);
				return;
			}
			int selectedRowIndex = phraseTable.getSelectedRows()[0];
			int targetColumnIndex = ptModel.findColumn(Constant.FACTOR_TOTAL_SENTENCES);
			if("0".equals(ptModel.getValueAt(selectedRowIndex, targetColumnIndex).toString())){
				JOptionPane.showMessageDialog(null, Constant.ERROR_MSG_NO_DATA_CAN_EXPORT, Constant.ERROR_MSG_ERROR, JOptionPane.ERROR_MESSAGE);
				return;
			}
			exportExperData();
		}
	}
	
	private void exportExperData(){
		JFileChooser saveFileDialog = new JFileChooser();
		int result = saveFileDialog.showSaveDialog(this);
		if(JFileChooser.APPROVE_OPTION == result){
			String savePath = saveFileDialog.getSelectedFile().getPath();			
			File file = new File(savePath);
			if(file.exists()){
				file.delete();
			}
			try {
				PrintWriter pw = new PrintWriter(file);
				pw.write(controller.export(phraseTable));
				pw.flush();
				pw.close();
				
				JOptionPane.showMessageDialog(null, Constant.MSG_DATA_EXPORTED_SUCCESSFUL, "", JOptionPane.INFORMATION_MESSAGE);
			} catch (FileNotFoundException error) {
				error.printStackTrace();
			}
		}
	}
	
	private void fireTableModelChanged(){
		ptModel.fireTableDataChanged();
		ptModel.fireTableChanged(null);
	}
	
	public void refresh() {
		setTableModel();
		fireTableModelChanged();
		phraseTable.removeColumn(phraseTable.getColumn(Constant.ID));
	}
}
