package main.setting;

import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.io.File;
import java.util.ArrayList;
import java.util.Locale;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.JComboBox;
import javax.swing.JLabel;

import tools.util.Constant;
import tools.util.I18NComboBoxRenderer;

public class LanguagePanel extends SettingObserverPanel implements ItemListener {

	private static final long serialVersionUID = -1151060068876668866L;
	private JLabel lbLang;
	private JComboBox cbLang;
	private SettingController controller;
	
	public LanguagePanel(SettingController controller) {
		this.controller = controller;
		createView();
	}
	
	private void createView(){
		//xxx i18n
		setBorder(BorderFactory.createTitledBorder("Language Setting"));
		
		Box langBox = Box.createHorizontalBox();
		lbLang = new JLabel(Constant.LANGUAGE);
		
		ArrayList<Locale> localeList = new ArrayList<Locale>();
		
		File file = new File(System.getProperty("user.dir") + System.getProperty("file.separator") + Constant.LANG_PATH);
		String[] list = file.list();
		
		for(int i = 0; i < list.length; i++){
			if(list[i].endsWith(".properties") && list[i].contains("_")){
				String[] sp = list[i].split("_");
				int loc = sp[2].lastIndexOf(".");
				localeList.add(new Locale(sp[1], sp[2].substring(0,loc)));
			}
		}
		
		cbLang = new JComboBox(localeList.toArray());
		I18NComboBoxRenderer renderer = new I18NComboBoxRenderer();
		cbLang.setRenderer(renderer);
		initParameter();
		cbLang.addItemListener(this);
		langBox.add(lbLang);
		langBox.add(cbLang);
		
		add(langBox);
	}
	
	/**
	 * 先檢查資料庫有無該 setting row，有就取出值並設定對應的 ComboBox Item，
	 * 否則新增該筆資料。
	 */
	private void initParameter() {
		String locStr = controller.initParameter(Constant.LANGUAGE_SETTING_NAME);
		if(locStr.contains("_")){
			String[] sp = locStr.split("_");
			cbLang.setSelectedItem(new Locale(sp[0], sp[1]));
		}
	}
	
	public void itemStateChanged(ItemEvent e) {
		if(cbLang.equals(e.getSource())){
			notifyBtnEnable();
			controller.setLocale((Locale)cbLang.getSelectedItem());
		}
	}
}
