package main.setting;

import java.awt.Dimension;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import tools.util.Constant;
import vo.MailSettingData;
/**
 * Mail 相關參數設置的 Panel。
 * @author pkwbim&Aries
 *
 */
public class MailSettingPanel extends SettingObserverPanel implements ItemListener, DocumentListener {

	private static final long serialVersionUID = 5081055394685625906L;
	private SettingController controller;
	private JTextField smtpText;
	private JTextField portText;
	private JCheckBox sslCb;
	private JTextField mailText;
	private JTextField nameText;
	private JTextArea content;
	
	public MailSettingPanel(SettingController controller) {
		this.controller = controller;
		createView();
	}
	
	public void createView(){
		//xxx i18n
		setBorder(BorderFactory.createTitledBorder("Mail Setting"));
		
		JLabel contentLab = new JLabel(Constant.MAIL_TEMPLATE_LAB);
		JLabel smtpLab = new JLabel(Constant.SMTP_LAB);
		JLabel portLab = new JLabel(Constant.PORT_LAB);
		JLabel mailLab = new JLabel(Constant.MAIL_ADDRESS_LAB);
		JLabel nameLab = new JLabel(Constant.MAIL_USERNAME_LAB);
		
		smtpText = new JTextField(10);
		portText = new JTextField(10);
		sslCb = new JCheckBox(Constant.SSL_LAB);
		mailText = new JTextField(10);
		nameText = new JTextField(10);
		
		content = new JTextArea();
		JScrollPane jsPane = new JScrollPane(content);
		jsPane.setPreferredSize(new Dimension(200, 200));
		
		initParameter();
		
		smtpText.getDocument().addDocumentListener(this);
		portText.getDocument().addDocumentListener(this);
		mailText.getDocument().addDocumentListener(this);
		nameText.getDocument().addDocumentListener(this);
		content.getDocument().addDocumentListener(this);
		sslCb.addItemListener(this);
		
		Box box = Box.createVerticalBox();
		Box b1 = Box.createHorizontalBox();
		Box b2 = Box.createHorizontalBox();
		Box b3 = Box.createHorizontalBox();
		
		b1.add(smtpLab);
		b1.add(smtpText);
		b1.add(portLab);
		b1.add(portText);
		b1.add(sslCb);
		b2.add(mailLab);
		b2.add(mailText);
		b3.add(nameLab);
		b3.add(nameText);
		
		box.add(b1);
		box.add(b2);
		box.add(b3);
		box.add(contentLab);
		box.add(jsPane);
		
		this.add(box);
	}
	
	/**
	 * 將各項設定值轉成 MailSettingData Object。
	 */
	public MailSettingData setMailSettingData(){
		MailSettingData data = new MailSettingData();
		data.setName(nameText.getText());
		data.setPort(portText.getText());
		data.setSender(mailText.getText());
		data.setSmtp(smtpText.getText());
		data.setContent(content.getText());
		if(sslCb.isSelected()){
			data.setSsl("1");
		}else{
			data.setSsl("0");
		}
		
		return data;
	}
	
	/**
	 * 先檢查資料庫有無該 setting row，有就取出值並設定對應的 TextField，
	 * 否則新增該筆資料。
	 */
	protected void initParameter(){
		content.setText(controller.initParameter(Constant.CONTENT));
		smtpText.setText(controller.initParameter(Constant.SMTP));
		portText.setText(controller.initParameter(Constant.PORT));
		if("1".equals(controller.initParameter(Constant.SSL))){
			sslCb.setSelected(true);
		}
		mailText.setText(controller.initParameter(Constant.MAIL));
		nameText.setText(controller.initParameter(Constant.MAIL_USERNAME));
	}
	
	public void changedUpdate(DocumentEvent e) {
	}

	public void insertUpdate(DocumentEvent e) {
		notifyBtnEnable();
	}

	public void removeUpdate(DocumentEvent e) {
		notifyBtnEnable();
	}
	
	public void itemStateChanged(ItemEvent arg0) {
		notifyBtnEnable();
	}
}
