package main.setting;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.io.File;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JRadioButton;
import javax.swing.JTextField;

import tools.util.Constant;
import vo.RData;

public class RSettingPanel extends SettingObserverPanel implements ActionListener, ItemListener  {

	private static final long serialVersionUID = 2075972966955061803L;
	private SettingController controller;
	private JRadioButton defBtn;
	private JRadioButton custBtn;
	private JTextField custText;
	private JTextField rText;
	private JTextField resultText;
	private JButton rBtn;
	private JButton resultBtn;
	private JButton custOkBtn;
	private JCheckBox[] cBox;
	
	public RSettingPanel(SettingController controller) {
		this.controller = controller;
		createView();
	}
	
	public void createView(){
		//xxx i18n
		setBorder(BorderFactory.createTitledBorder("R Setting"));

		rText = new JTextField(5);
		rBtn = new JButton(Constant.SETTING_SET);
		JLabel rLabel = new JLabel(Constant.SETTING_RDIRECTORY);
		
		resultText = new JTextField(5);
		resultBtn = new JButton(Constant.SETTING_SET);
		JLabel resultLabel = new JLabel(Constant.SETTING_SAVE_RESULT);
		resultText.setEditable(false);
		
		defBtn = new JRadioButton(Constant.SETTING_R_DEFAULT);
		custBtn = new JRadioButton(Constant.SETTING_R_CUSTOM);
		defBtn.setSelected(true);
		
		defBtn.addItemListener(this);
		custBtn.addItemListener(this);
		
		ButtonGroup bg = new ButtonGroup();
		bg.add(defBtn);
		bg.add(custBtn);
		
		custText = new JTextField(5);
		custOkBtn = new JButton(Constant.SETTING_SET);
		cBox = new JCheckBox[2];
		cBox[0] = new JCheckBox(Constant.RX0);
		cBox[1] = new JCheckBox(Constant.R0X);
		custText.setEditable(false);
		custOkBtn.setEnabled(false);
		
		//XXX 暫時禁能 
		cBox[1].setEnabled(false);
		
		resultBtn.addActionListener(this);
		custOkBtn.addActionListener(this);
		
		Box box = Box.createVerticalBox();
		Box defBox = Box.createHorizontalBox();
		Box defItemBox = Box.createVerticalBox();
		Box custBox = Box.createHorizontalBox();
		Box resultBox = Box.createHorizontalBox();
		
		resultBox.add(resultLabel);
		resultBox.add(resultText);
		resultBox.add(resultBtn);
		
		defBox.add(defBtn);
		defBox.add(defItemBox);
		defBox.add(Box.createHorizontalGlue());
		for(JCheckBox c : cBox){
			defItemBox.add(c);
			c.addItemListener(this);
		}
		
		custBox.add(custBtn);
		custBox.add(custText);
		custBox.add(custOkBtn);

		if(System.getProperty("os.name").toLowerCase().startsWith("windows")){
			Box rb = Box.createHorizontalBox();
			rText.setEditable(false);
			rBtn.addActionListener(this);
			rb.add(rLabel);
			rb.add(rText);
			rb.add(rBtn);
			box.add(rb);
		}
		box.add(resultBox);
		box.add(defBox);
		box.add(custBox);

		add(box);
		
		initFileParameter();
	}
	
	/**
	 * 初始化 R 相關路徑。<br />
	 * 檢查 Setting Table 有無與 r 相關的 row，有則設定畫面，無則新增 r row。<br />
	 */
	private void initFileParameter(){
		controller.initParameter(Constant.RFILE);
		//新增家目錄 row
		controller.initParameter(Constant.HOME);
		//取得家目錄路徑
		controller.update(Constant.HOME, System.getProperty("user.home"));
		rText.setText(controller.initParameter(Constant.R));
		resultText.setText(controller.initParameter(Constant.RRESULT));
		String type = controller.initParameter(Constant.RTYPE);
		if("".equals(type)){
			controller.update(Constant.RFILE, Constant.R_SCRIPT_ARRAY[0]);
		}
		for(int i = 0; i < type.length(); i++){
			if("1".equals(type.substring(i,1))){
				cBox[i].setSelected(true);
			}
		}
	}
	
	/**
	 * 顯示 FileChooseDialog，並將選擇後的檔案路徑設定到 TextField 中。
	 * @param text 所要設定的 JTextField
	 * @param canChooseFile 是否可以選取檔案。
	 */
	private void selectFile(JTextField text, boolean canChooseFile){
		JFileChooser fc = new JFileChooser();
		if(canChooseFile){
			fc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
		}
		fc.setCurrentDirectory(new File(System.getProperty("user.dir")));
		int returnVal = fc.showOpenDialog(this);
		if(returnVal == JFileChooser.APPROVE_OPTION){
			text.setText(fc.getSelectedFile().toString());
		}
	}
	
	public boolean isSelectedR(){
		return defBtn.isSelected() || custBtn.isSelected();
	}
	
	private String getRScriptType(){
		String type = "";
		for(int i = 0; i < cBox.length; i++){
			if(cBox[i].isSelected()){
				type += "1";
			}else{
				type += "0";
			}
		}
		return type;
	}
	
	public RData getRSettingData(){
		RData data = new RData();
		data.setType(getRScriptType());
		data.setR(rText.getText());
		if(defBtn.isSelected()){
			data.setRFile(Constant.R_SCRIPT_ARRAY[Integer.parseInt(data.getType(), 2)]);
		}else{
			data.setRFile(custText.getText());
		}
		data.setrResult(resultText.getText());
		return data;
	}

	public void actionPerformed(ActionEvent e) {
		if(custOkBtn.equals(e.getSource())){
			selectFile(custText, false);
		}else if(rBtn.equals(e.getSource())){
			selectFile(rText, false);
		}else if(resultBtn.equals(e.getSource())){
			selectFile(resultText, true);
		}
		notifyBtnEnable();
	}
	
	public void itemStateChanged(ItemEvent e) {
		if(defBtn.equals(e.getSource())){
			cBox[0].setEnabled(defBtn.isSelected());
			cBox[1].setEnabled(defBtn.isSelected());
			//XXX 暫時禁能 
			cBox[1].setEnabled(false);
			custText.setText("");
		}else if(custBtn.equals(e.getSource())){
			custOkBtn.setEnabled(custBtn.isSelected());
			for(JCheckBox c : cBox){
				c.setSelected(false);
			}
		}
		notifyBtnEnable();
	}
}
