package main.setting;

import java.util.Iterator;
import java.util.List;
import java.util.Locale;

import javax.swing.JOptionPane;

import org.springframework.validation.BindException;
import org.springframework.validation.BindingResult;
import org.springframework.validation.ObjectError;

import model.SettingModel;
import tools.util.Constant;
import tools.validation.MailSettingValidation;
import vo.MailSettingData;
import vo.RData;
import vo.SettingData;

public class SettingController {
	
	private SettingModel model;
	private Locale locale;
	
	public SettingController(SettingModel model){
		this.model = model;
	}
	
	public String initParameter(String name){
		SettingData data = new SettingData();
		if(model.isRowExist(name)){
			data = model.select(name);
		}else{
			model.insert(name, "");
			data.setName(name);
			data.setValue("");
		}
		return data.getValue();
	}
	
	public int update(String name, String value){
		return model.update(name, value);
	}

	public void setLocale(Locale locale) {
		this.locale = locale;
	}
	
	public void setLanguage(){
		if(locale != null){
			int i = update(Constant.LANGUAGE_SETTING_NAME, locale.getLanguage()+ "_" + locale.getCountry());
			
			boolean isApply = false;
			if(1 == i){
				isApply = true;
			}
			//show message
			if(isApply){
				JOptionPane.showMessageDialog(null, Constant.MSG_RESTART);
			}else {
				JOptionPane.showMessageDialog(null, Constant.ERROR_RESTART);
			}
		}
	}

	public void setMailSeeting(MailSettingData data) {
		MailSettingValidation validation = new MailSettingValidation();
		BindException errors = new BindException(data, "MailSettingData");
		validation.validate(data, errors);
		
		if(errors.getErrorCount()>0){
			BindingResult rs = errors.getBindingResult();
			List<ObjectError> list = rs.getAllErrors();
			Iterator<ObjectError> iterator = list.iterator();
			String errorMsg = "";
			while(iterator.hasNext()){
				ObjectError error = (ObjectError)iterator.next();
				errorMsg += error.getDefaultMessage() + System.getProperty("line.separator");
			}
			JOptionPane.showMessageDialog(null, errorMsg, Constant.WARNING_MSG_WARNING, JOptionPane.WARNING_MESSAGE);
		}else{
			updateMailSetting(data);
		}
	}
	
	private void updateMailSetting(MailSettingData data){
		update(Constant.CONTENT, data.getContent());
		update(Constant.SMTP, data.getSmtp());
		update(Constant.PORT, data.getPort());
		update(Constant.SSL, data.getSsl());
		update(Constant.MAIL, data.getSender());
		update(Constant.MAIL_USERNAME, data.getName());
	}
	
	public void setServerURL(String url){
		if(!url.startsWith("http://")){
			JOptionPane.showMessageDialog(null, Constant.WARNING_MSG_URL_MAL_FORMED, Constant.WARNING_MSG_WARNING, JOptionPane.WARNING_MESSAGE);
		}else{
			//檢查網址是否以 '/' 結尾，不是的話要加上 '/'。
			String server = url ;
			if(!url.endsWith("/")){
				server += "/";
			}
			update(Constant.SERVER, server);
		}
	}

	public void setR(RData data) {
		update(Constant.RTYPE, data.getType());
		update(Constant.RFILE, data.getRFile());
		update(Constant.R, data.getR());
		update(Constant.RRESULT, data.getrResult());
	}
}
