package model;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.PreparedStatementCreator;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.jdbc.support.GeneratedKeyHolder;
import org.springframework.jdbc.support.KeyHolder;
import org.springframework.jdbc.support.rowset.SqlRowSet;

import tools.db.DAO;
import vo.CachePhraseData;

public class CachePhraseModel {
	
	private static String CACHE_TABLE = "cache_phrase";
	private static String[] CACHE_PHRASE = {"id", "phrase_description", "experimental_sentence", "phrase_id", "phrase_count", "questionexper_id"};
	private static JdbcTemplate jdbcTemplate = DAO.getJdbcTemplate();
	
	public CachePhraseModel(){}
	
	public Number insert(final CachePhraseData data){
		
		KeyHolder keyHolder = new GeneratedKeyHolder();
		jdbcTemplate.update(new PreparedStatementCreator(){
			public PreparedStatement createPreparedStatement(Connection conn)
					throws SQLException {
				String sql = "insert into "+ CACHE_TABLE +" values(null, ?, ?, ?, ?, ?);";
				PreparedStatement ps = conn.prepareStatement(sql);
				ps.setObject(1, data.getPhraseDescription());
				ps.setObject(2, data.getExperimentalSentence());
				ps.setObject(3, data.getPhraseId());
				ps.setObject(4, data.getPhraseCount());
				ps.setObject(5, data.getQuestionexperId());
	            return ps;
			}}, keyHolder);
		return keyHolder.getKey();
	}
	
	/**
	 * 傳入實驗id,回傳該實驗所儲存的句庫與實驗
	 * @param questionexper_id
	 * @return
	 */
	public static ArrayList<String> getExperimentalSentences(int experId) {
		ArrayList<String> ret = new ArrayList<String>();
		SqlRowSet rs = DAO.select(String.format(
				"select * from %s where questionexper_id=?;", 
				CACHE_TABLE), experId);
		while(rs.next()){
			ret.add(rs.getString(CACHE_PHRASE[2]));
		}
		return ret;
	}

	public static Object getDescription(int factorId) {
		SqlRowSet rs = DAO.select(String.format("select %s from %s where %s=?;",
				CACHE_PHRASE[1], CACHE_TABLE, CACHE_PHRASE[3]), factorId);
		if(rs.next()){
			return rs.getString(CACHE_PHRASE[1]);
		}
				
		return "";
	}

	public static Object getExperNum(int id) {
		String sql = "select count(*) from " + CACHE_TABLE + " where questionexper_id = ?;";
		return jdbcTemplate.queryForInt(sql, id);
	}
	
	public int getFactorCount(int id){
		String sql = "select phrase_count from " + CACHE_TABLE + " where questionexper_id = ?;";
		SqlRowSet rs = jdbcTemplate.queryForRowSet(sql, id);
		int count = 0;
		if(rs.next()){
			count = rs.getInt(1) / 10;
		}
		return count;
	}
	
	public static ArrayList<String> getExperDataExperId(int id){
		String sql = "select * from " + CACHE_TABLE + " where questionexper_id = ?;";
		
		List<ArrayList<String>> dataList = jdbcTemplate.query(sql, new Object[]{id}, new RowMapper<ArrayList<String>>(){
			public ArrayList<String> mapRow(ResultSet rs, int i)
					throws SQLException {
				ArrayList<String> list = new ArrayList<String>();
				do{
					list.add(rs.getString("experimental_sentence"));
				}while(rs.next());
				return list;
			}
		});
		
		ArrayList<String> experData;
		if(dataList.size() == 0){
			experData = new ArrayList<String>();
		}else{
			experData = dataList.get(0);
		}
		return experData;
	}
}
