package model;

import java.util.ArrayList;

import javax.swing.table.AbstractTableModel;

import tools.util.Constant;

// this is the entry point to manage all the experiments, questionnaires and questions.
// and can be designed to complaint Singleton Design Pattern or Static methods/class
// but which one is more proper is a myth.

/**
 * 此 class 是使用這整個 實驗-問卷-問題 model 的進入點，也就是所謂的『實驗管理者』。
 * 同時實作/提供 TableModel 的介面，讓使用者易於搭配 Java Swing 使用 -- 直接放入 table 中
 * 即可。
 * 由於在這個 class 載入的時候會去清除 問題的 cache ，如果因此造成 database lock 等的 exception，
 * 可以把他拿掉。(將以下 static init block 註解掉)
 */
@SuppressWarnings("serial")
public class ExperimentsTableModel extends AbstractTableModel {

	static {
		// 偷偷的清除 Question 的快取
		Question.vacuum();
	}
	
	private static ExperimentsTableModel singleton = new ExperimentsTableModel();

	public static ExperimentsTableModel getInstance() {
		return singleton;
	}

	private ExperimentsTableModel() {
	}
	
	// -----------------------------------------------------------------
	
	private ArrayList<Experiment> experiments = Experiment.restoreAllExperiments();

	public ArrayList<Experiment> getAllExperiments() {
		return experiments;
	}

	// -----------------------------------------------------------------
	
	private static String[] COL_NAMES = {"id", "title", "factor desc.", "date", "guide title", "fake question num", "exper num", "question num"};
	@Override
	public int getColumnCount() {
		return COL_NAMES.length;
	}

	private static String[] I18N_COL_NAMES = {Constant.NUMOF, Constant.EXP_EXP_TITLE, Constant.EXP_EXP_FACTOR,
											Constant.EXP_EXP_DATE, Constant.EXP_GUIDE, Constant.EXP_EXP_FAKE_QUE_NUM,
											Constant.EXP_EXP_EXPER_QUE_NUM, Constant.EXP_EXP_QUE_NUM};
	@Override
	public String getColumnName(int column) {
		return I18N_COL_NAMES[column];
	}

	@Override
	public int getRowCount() {
		return experiments.size();
	}

	@Override
	public Object getValueAt(int arg0, int arg1) {
		if(arg1 == 0){
			return arg0+1;
		}
		
		Experiment e = experiments.get(arg0);
		return getValueAt(e, arg1);
	}

	private Object getValueAt(Experiment e, int col) {
		if ("id".equals(COL_NAMES[col])) {
			return e.getId();
		} else if ("title".equals(COL_NAMES[col])) {
			return e.getTitle();
		} else if ("factor desc.".equals(COL_NAMES[col])) {
			return CachePhraseModel.getDescription(e.getFactorId());
		} else if ("fake question num".equals(COL_NAMES[col])) {
			return e.getFakeQuestionNum();
		} else if ("question num".equals(COL_NAMES[col])) {
			return e.getQuestionnaires().size();
		} else if ("guide title".equals(COL_NAMES[col])) {
			return e.getGuideTitle();
		} else if ("date".equals(COL_NAMES[col])) {
			return e.getDate();
		} else if ("exper num".equals(COL_NAMES[col])){
			return CachePhraseModel.getExperNum(e.getId());
		}
		return null;
	}
	
	public Object getId(int i){
		Experiment e = experiments.get(i);
		return getValueAt(e, 0);
	}
}
