package model;

import java.util.List;

import javax.swing.table.AbstractTableModel;

import org.springframework.dao.IncorrectResultSizeDataAccessException;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.support.rowset.SqlRowSet;

import tools.db.DAO;
import tools.db.DBConnector;
import tools.util.Constant;

/**
 * 干擾句
 */
@SuppressWarnings("serial")
public class FakeQuestionsModel extends AbstractTableModel {

	static final String TABLE = "interference";
	static final String[] COL_NAMES = {"id", "interference"};
	
	private static final FakeQuestionsModel singleton = new FakeQuestionsModel();

	/**
	 * 取得干擾句的建構子
	 * @return 干擾句本身
	 */
	public static FakeQuestionsModel getTableModel() {
		return singleton;
	}

	private FakeQuestionsModel() {
	}

	// -----------------------------------------------------------------
	/**
	 * 新增干擾句
	 * @param question 干擾句句子內容
	 * @return insert 結果。 -1 => Error or 已有存在的值 , 若為其他值則為該筆新增資料的ID
	 */
	public static int insert(String question) {
		int id = FakeQuestionsModel.getId(question);
		if (-1 != id) {
			return -1;
		}

//		int i = DAO.insert(String.format("insert into %s values (null, '%s');", 
//				TABLE, question));
		int i = DAO.insert(TABLE, COL_NAMES, new Object[]{question});
		if (1 != i) {
			System.out.println("error on DAO.insert in FakeQuestionsModel.insert. (i=" + i + ")");
			return -1;
		}

		// @caution this depends on how you sort the data
		getTableModel().fireTableRowsInserted(0, 0);
		return getId(question);
	}
	
	/**
	 * 更新干擾句
	 * @param id 要更新的干擾句ID
	 * @param newQuestion 干擾句句子內容
	 * @return update結果。True, 更新成功。False, 更新失敗。
	 */
	public static boolean update(int id, String newQuestion) {
		int i = getId(newQuestion);
		if (-1 != i) {
			if(i == id){
				return true;
			}
			return false;
		}
		
		i = DAO.update(TABLE, new String[]{COL_NAMES[1]}, new Object[]{newQuestion}, id);
		if (1 != i) {
			System.out.println("error on DAO.update in FakeQuestionsModel.update. (i=" + i + ")");
			return false;
		}

		i = getTableModel().getIndex(id);
		getTableModel().fireTableRowsUpdated(i, i);
		return true;
	}
	
	/**
	 * 刪除干擾句
	 * @param id 要刪除的干擾句ID
	 * @return delete結果。True, 刪除成功。False, 刪除失敗。
	 */
	public static boolean delete(int id) {
		int index = getTableModel().getIndex(id);
		int i = DAO.delete(TABLE, id);
		if (1 != i) {
			System.out.println("error on DAO.delete in FakeQuestionsModel.delete (id=" + id + ", i=" + i + ")");
			return false;
		}

		getTableModel().fireTableRowsDeleted(index, index);
		return true;
	}

	// -----------------------------------------------------------------
	/**
	 * @param question 干擾句句子
	 * @return 干擾句ID
	 */
	public static int getId(String question) {
		JdbcTemplate jdbcTemplate = DBConnector.getJdbcTemplate();
		int ret = -1;
		try {
			ret = jdbcTemplate.queryForInt(
					String.format("select %s from %s where %s=?;", COL_NAMES[0], TABLE, COL_NAMES[1]), 
					(Object) question);
		} catch (IncorrectResultSizeDataAccessException e) {
		}
		return ret;
	}

	/**
	 * 取得干擾句句子
	 * @param id 要取得的干擾句ID
	 * @return 干擾句句子
	 */
	public static String getFakeQuestion(int id) {
		JdbcTemplate jdbcTemplate = DBConnector.getJdbcTemplate();
		try {
			return jdbcTemplate.queryForObject(String.format(
					"select %s from %s where %s=?;", COL_NAMES[1], TABLE,
					COL_NAMES[0]), new Object[] { id }, String.class);
		} catch (IncorrectResultSizeDataAccessException e) {

		}
		return null;
	}

	/**
	 * @return 資料庫內所有的干擾句
	 */
	public static List<String> getAllQuestions() {
		JdbcTemplate jdbcTemplate = DBConnector.getJdbcTemplate();
		return (List<String>) jdbcTemplate.queryForList(String.format("select %s from %s order by %s desc;", 
				COL_NAMES[1], TABLE, COL_NAMES[0]), String.class);
	}

	// useless at the moment
	// @return id of deleted data
//	public static int delete(String question) {
//		int id = getId(question);
//		boolean b = delete(id);
//		return true == b ? id : -1;
//	}

	// -----------------------------------------------------------------
	
	/**
	 * @param id 句子的ID
	 * @return 句子在干擾句內的Index
	 */
	public int getIndex(int id) {
		JdbcTemplate jdbcTemplate = DBConnector.getJdbcTemplate();
		List<Integer> ids = jdbcTemplate.queryForList(String.format("select %s from %s order by %s desc;", 
				COL_NAMES[0], TABLE, COL_NAMES[0]), Integer.class);
		int ret = 0;
		for (Integer i : ids) {
			if (id == i.intValue()) {
				return ret;
			}
			++ret;
		}
		return -1;
	}
	
	/**
	 * 取得該筆資料的干擾句ID
	 * @param i 該筆資料的index
	 * @return 干擾句ID
	 */
	// @param i index in the table model
	public int getId(int i) {
//		return Integer.parseInt((String)getValueAt(i, 0));
		JdbcTemplate jdbcTemplate = DBConnector.getJdbcTemplate();
		
		SqlRowSet rs = jdbcTemplate.queryForRowSet(String.format("select %s from %s order by %s desc;",
				COL_NAMES[0], TABLE, COL_NAMES[0]));
		while (rs.next()) {
			if (0 == i) {
				return rs.getInt(1);
			}
			--i;
		}
		return 0;
	}

	// -----------------------------------------------------------------
	
	//因為客戶要求不要顯示編號，所以-1欄
	@Override
	public int getColumnCount() {
		return COL_NAMES.length-1;
	}

	//因為客戶要求不要顯示編號，所以去掉Constant.NUMOF
//	private static final String[] I18N_COL_NAMES = {Constant.NUMOF, Constant.FAKE_QUE_FAKE_QUE};
	private static final String[] I18N_COL_NAMES = {Constant.FAKE_QUE_FAKE_QUE};
	@Override
	public String getColumnName(int column) {
		return I18N_COL_NAMES[column];
	}

	@Override
	public int getRowCount() {
		JdbcTemplate jdbcTemplate = DBConnector.getJdbcTemplate();
		return jdbcTemplate.queryForInt(String.format("select count(*) from %s;", TABLE));
	}

	//干擾句編輯的地方有用到這個function, 所以修改時需要注意一下
	@Override
	public Object getValueAt(int arg0, int arg1) {
//		if(arg1 == 0){
//			return arg0+1;
//		}
		//因為客戶要求不要顯示編號，所以+1從第二個值開始取
		arg1 += 1;
		
		JdbcTemplate jdbcTemplate = DBConnector.getJdbcTemplate();
		
		SqlRowSet rs = jdbcTemplate.queryForRowSet(String.format("select %s from %s order by %s desc;",
				COL_NAMES[arg1], TABLE, COL_NAMES[0]));
		while (rs.next()) {
			if (0 == arg0) {
				return rs.getString(1);
			}
			--arg0;
		}
		return null;
	}

	/**
	 * 取得干擾句總數
	 * @return 干擾句總數
	 */
	public static int getFakeQuestionNum(){
		return singleton.getRowCount();
	}
}
