package model;

import java.security.Security;
import java.sql.Date;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;

import javax.mail.Authenticator;
import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.PasswordAuthentication;
import javax.mail.Session;
import javax.mail.Transport;
import javax.mail.internet.AddressException;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeMessage;

import org.springframework.jdbc.core.BatchPreparedStatementSetter;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.jdbc.support.rowset.SqlRowSet;

import tools.db.DAO;
import vo.MailInfo;
import vo.MailToServerRowMapper;
import vo.SurveyResult;
import vo.UploadQuestionaireData;

import com.sun.net.ssl.internal.ssl.Provider;

public class LittleOne {

	private JdbcTemplate jdbcTemplate = DAO.getJdbcTemplate();
	/**
	 * 取得上傳所需的資料(肯定、否定、指導語等資料)
	 * 
	 * @param questionnaire_id
	 * @param questionaireUid
	 * @return
	 */
	public UploadQuestionaireData getUploadQuestionaireData(int questionnaire_id) {
		UploadQuestionaireData ret = new UploadQuestionaireData();
		String sql = "select questionexper.positive, questionexper.negative, questionexper.guidesentence from questionnaire inner join questionexper on questionnaire.questionexper_id = questionexper.id where questionnaire.id = ?";
		SqlRowSet rs = jdbcTemplate.queryForRowSet(sql, questionnaire_id);
		if (rs.next()) {
			ret.setPositive(rs.getString("positive"));
			ret.setNegative(rs.getString("negative"));
			ret.setGuidesentence(rs.getString("guidesentence"));
		}
		ArrayList<MailToServerRowMapper> m2s = getMailToServerData(questionnaire_id);
		ret.setExperSentences(m2s.toArray(new MailToServerRowMapper[m2s.size()]));
		return ret;
	}

	private ArrayList<MailToServerRowMapper> getMailToServerData(int id) {
		String sql1 = "select questionsentence.questionsentence, questioncontent.is_interference  , questioncontent.id from questionnaire "
				+ "join questioncontent on questionnaire.id = questioncontent.question_id "
				+ "join questionsentence on questioncontent.questionsentence_id = questionsentence.id "
				+ "where questionnaire.id = ?";
		final ArrayList<MailToServerRowMapper> m2s = new ArrayList<MailToServerRowMapper>();
		// new Object[]{} 內的值，為使用者選取的值
		jdbcTemplate.query(sql1, new Object[] { id },
				new RowMapper<ResultSet>() {
					public ResultSet mapRow(ResultSet rs, int arg1)
							throws SQLException {
						do {
							MailToServerRowMapper m2 = new MailToServerRowMapper(
									rs);
							m2s.add(m2);
						} while (rs.next());

						return null;
					}
				});
		return m2s;
	}

	/**
	 * 在miniJudge執行寄送問卷
	 * 
	 * @throws AddressException
	 * @throws MessagingException
	 */
	public void mailsend(MailInfo info) throws AddressException, MessagingException {
		Security.addProvider(new Provider());
		// Get a Properties object
		Properties props = System.getProperties();
		props.setProperty("mail.smtp.host", info.getSmtp()); // 設定SMTP
		// Server

		// SSL
		if (info.isSsl()) {
			final String SSL_FACTORY = "javax.net.ssl.SSLSocketFactory";
			props.setProperty("mail.smtp.socketFactory.class", SSL_FACTORY);
			props.setProperty("mail.smtp.socketFactory.fallback", "false");
			props.setProperty("mail.smtp.port", info.getPort());				//port :465
			props.setProperty("mail.smtp.socketFactory.port", info.getPort());
		}
		props.put("mail.smtp.auth", "true"); // 是否使用認証
		final String username = info.getName(); // 寄信的帳號
		final String password = info.getPassword(); // 寄信的密碼
		Session session = Session.getInstance(props,
				new Authenticator() {
					protected PasswordAuthentication getPasswordAuthentication() {
						return new PasswordAuthentication(username, password);
					}
				});

		// -- Create a new message --
		Message msg = new MimeMessage(session);
		msg.setFrom(new InternetAddress(info.getSender())); // 寄件者
		msg.setRecipients(Message.RecipientType.TO, InternetAddress.parse(info.getReciver(), false)); // 收件者
		msg.setSubject(info.getSubject()); // 郵件標題
		msg.setText(info.getContext()); // 郵件內文
		msg.setSentDate(new java.util.Date());
		Transport.send(msg);
		System.out.println("Message sent.");
	}

	// TODO 由資料庫取得factor的個數
	public int getFactorCount() {
		return 0;
	}
	
	public ArrayList<String> getUid(){
		ArrayList<String> ret = new ArrayList<String>();
		String sql = "select uuidmapping.uuid from uuidmapping join questionnaire on uuidmapping.questionnaire_id = questionnaire.id where questionnaire.state = 'SENT';";
		SqlRowSet rs = jdbcTemplate.queryForRowSet(sql);
		while(rs.next()){
			ret.add(rs.getString("uuid"));
		}
		return ret;
	}
	
	/**
	 * 下載實驗結果。
	 * @param List<SurveyResult> list 從 server 端載回來的資料。
	 */
	public void saveDownloadResult(List<SurveyResult> list){
		updateQuestionsAnwser(list);
//		updateQuestionsExperDate(list);
		fireTable(list);
	}
	
	/**
	 * 下載後 ，更新問卷的顯示畫面
	 * @param list
	 */
	private void fireTable(List<SurveyResult> list) {
		//將要更新的問卷跟時間另外存一份uidList;
		ArrayList<ArrayList<String>> uidList = new ArrayList<ArrayList<String>>();
		for(SurveyResult l : list){
			ArrayList<String> s = new ArrayList<String>();

			if((uidList.size() == 0)){
				s.add(l.getUuid());
				s.add(l.getExperDate());
				uidList.add(s);
			}else if(!l.getUuid().equals(uidList.get(uidList.size()-1).get(0))){
				s.add(l.getUuid());
				s.add(l.getExperDate());
				uidList.add(s);
			}
		}
		//取得Questionnaire ID for FireTable
		String sql = "select questionnaire_id from uuidmapping where uuid = ?;";
		for(ArrayList<String> s : uidList){
			SqlRowSet rs = jdbcTemplate.queryForRowSet(sql, new Object[]{s.get(0)});
			if(rs.next()){
				s.set(0,  rs.getString("questionnaire_id"));
			}
		}
		
		//取得該筆問卷的記憶體資料才有辦法Fire
		ArrayList<Experiment> experiments = ExperimentsTableModel.getInstance().getAllExperiments();
		for(Experiment e : experiments){
			ArrayList<Questionnaire> questionnaires = e.getQuestionnaires();
			for(Questionnaire q : questionnaires){
				for(ArrayList<String> s : uidList){
					if(s.get(0).equals(q.getId()+"")){
						q.setState(Questionnaire.State.COMPLETED);
						q.setAnswerDate(Date.valueOf(s.get(1)));
					}
				}
			}
		}
	}
	
	/**
	 * 將從 server 端下載回來的問卷答案存入 client 端的 DB。
	 * @param List<SurveyResult> list 從 server 端載回來的資料。
	 * @return int[] 更新的結果。
	 */
	protected int[] updateQuestionsAnwser(final List<SurveyResult> list){
		String sql = "update questioncontent set answer = ? where id = ?;";
		int[] result = jdbcTemplate.batchUpdate(sql, new BatchPreparedStatementSetter() {
			public void setValues(PreparedStatement ps, int i) throws SQLException {
				ps.setInt(1, list.get(i).getAnswer());
				ps.setInt(2, list.get(i).getQuestionId());
			}
			
			public int getBatchSize() {
				return list.size();
			}
		});
		
		return result;
	}
}
