package model;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;

import org.springframework.jdbc.support.rowset.SqlRowSet;

import tools.db.DAO;

/**
 * 問題
 */
public class Question {

	static String TABLE = "questioncontent";
	static String[] COL_NAMES = {"id", "question_id", "questionsentence_id", "is_interference", "answer"};
	
	private int id = -1;
	private int cacheId;
	private Questionnaire questionnaire;
	private boolean isFake;
	private boolean answer;
	
	/**
	 * 從資料庫復原所屬的問題
	 * 注意：請勿直接呼叫此 method
	 */
	public static ArrayList<Question> restoreQuestions(Questionnaire questionnaire) {
		ArrayList<Question> ret = new ArrayList<Question>();
		try {
			ResultSet rs = DAO.select(String.format("select * from %s where %s='%s';", 
					TABLE, COL_NAMES[1], questionnaire.getId()));
			while (rs.next()) {
				Question q = new Question(questionnaire, Boolean.parseBoolean(rs.getString(4)), rs.getInt(3));
				
				q.id = rs.getInt(1);
				q.answer = Boolean.parseBoolean(rs.getString(5));

				ret.add(q);
			}
		} catch (SQLException e) {
			e.printStackTrace();
			throw new RuntimeException("Question.restoreQuestions: e.getMessage=" + e.getMessage());
		} finally {
			DAO.close();
		}
		return ret;
	}

	private Question(Questionnaire q, boolean isFake, int cacheId) {
		this.questionnaire = q;
		this.isFake = isFake;
		this.cacheId = cacheId;
	}
	
	/**
	 * 請勿直接呼叫此 method。
	 * 一般情況下，沒可能直接新增問題，
	 * 通常是新增問卷，而問卷會負責處理『問題』
	 */
	public Question(String s, boolean isFake, Questionnaire questionnaire) {
		this(questionnaire, isFake, Question.getCacheId(s));
		insert();
		questionnaire.getQuestions().add(this);
	}

	/**
	 * 請勿使用此 method，此 method 為搭配 constructor 使用。
	 * 此 method 的目的為：存入資料庫，並取得『自動新增』的 id
	 */
	private int insert() {
		assert(-1 == 0);
			
		int i = DAO.insert(String.format("insert into %s values (null, '%s', '%s', '%s', '%s');", 
				TABLE, questionnaire.getId(), cacheId, isFake, answer));
		if (1 != i) {
			throw new RuntimeException("question.insert: DAO.insert");
		}
		
		try {
			ResultSet rs = DAO.select(String.format("select %s from %s  order by %s desc limit 1;", COL_NAMES[0], TABLE, COL_NAMES[0]));
			rs.next();
			id = rs.getInt(1);
		} catch (SQLException e) {
			//e.printStackTrace();
			throw new RuntimeException("question.insert: e.getMessage=" + e.getMessage());
		} finally {
			DAO.close();
		}
		return id;
	}

	/**
	 * 刪除此問題。不應直接呼叫，而應該透過刪除問卷來刪除之。
	 */
	void delete() {
		// remove from memory
		questionnaire.getQuestions().remove(this);
		// remove from database
		int i = DAO.delete(String.format("delete from %s where %s='%s';", 
				TABLE, COL_NAMES[0], id));
		if (1 != i) {
			throw new RuntimeException("Error in delete a question. id=" + id);
		}
	}

	/**
	 * 當資料有更新，自動更新至資料庫
	 */
	private void sync() {
		assert(id >= 0);
		
		int i = DAO.update(String.format("update %s set %s='%s' where %s='%s';", 
				TABLE, COL_NAMES[4], answer, COL_NAMES[0], id));
		if (1 != i) {
			throw new RuntimeException("Error in sync a question. id=" + id);
		}
	}

	public void setAnswer(boolean answer) {
		this.answer = answer;
		sync();
	}

// ---------------------------------------------------------------
	
	public boolean isFake() {
		return isFake;
	}

	public boolean getAnswer() {
		return answer;
	}

	public int getId() {
		return id;
	}

	/**
	 * @return 問題的問句
	 */
	public String getQuestion() {
		return Question.getCacheString(cacheId);
	}

	public Questionnaire getQuestionnaire() {
		return questionnaire;
	}

	// ---------------------------------------------------------------
	
	/**
	 * @return 這個問題是屬於實驗中的『第幾份』問卷
     */
    public int getSpeaker() {
        return questionnaire.getExperiment().getQuestionnaires().indexOf(questionnaire) + 1;
    }

    /**
     * @return 這個問題是屬於問卷中的『第幾個』問題，注意：需去除干擾句
     */
    public int getOrder() {
    	int ret = 1;
    	ArrayList<Question> questions = questionnaire.getQuestions();
    	for (Question q : questions) {
    		if (q.isFake()) {
    			continue;
    		}
    		if (q == this) {
    			break;
    		}
    		++ret;
    	}
    	return ret;
    }

    /**
     * @return 答案。對轉為 1，錯轉為 0
     */
    public int getJudgement() {
    	return getAnswer() ? 1 : 0;
    }

	private int set = -1;
	private int item = -1;
	private int factor1 = 0;
	private int factor2 = 0;
	
	/**
	 * @return 這個問題是屬於『因素』中的『第幾個』句組
	 */
	public int getSet() {
		if (-1 == set) {
			setup();
		}
		return set;
	}

	/**
	 * 取得這個問題的原始順序？ 用來與打亂之後的順序相比較？ 或許在原本的架構底下是有意義的，但現在意義則不是那麼明確 目前以 id 代替 =>
	 * 沒有id!! 目前以 experimental.id * 4 + index(1~4) - 1
	 */
	public int getItem() {
		if (-1 == item) {
			setup();
		}
		return item;
	}

	/**
	 * 這個問題來自於 | Factor1 | Factor2 
	 * case1 | +1 | +1 
	 * case2 | +1 | -1 
	 * case3 | -1 | +1 
	 * case4 | -1 | -1 => 需要修改 table 將 experimental.id 存進去
	 */
	public int getFactor1() {
		if (0 == factor1) {
			setup();
		}
		return factor1;
	}

	public int getFactor2() {
		if (0 == factor1) {
			setup();
		}
		return factor2;
	}
	
	private void setup() {
		String content = getCacheString(cacheId);
		assert(null != content);
		
		//TODO 移除與句庫模組的關聯，這個未修改到，此時當句庫模組沒有該id時會出錯
		int expId = questionnaire.getExperiment().getId();
		//TODO 改成從cache_phrase取
		//如果句庫的句子移除時會一併刪除實驗的句子時則sets沒辦法以目前的方法取
		ArrayList<String> sets = CachePhraseModel.getExperDataExperId(expId);
		assert(null != sets);
		CachePhraseModel cp = new CachePhraseModel();
		
		int mod = 2;
		if (2 == cp.getFactorCount(expId)){
			mod = 4;
		}
		for(int i = 0; i < sets.size(); i++){
			if(sets.get(i).equals(content)){
				item = i + 1;
				set = i / mod + 1;
				int j = i % mod;
				
				switch(j){
					case 0:
						if(mod==4){
							factor1 = +1;
							factor2 = +1;
						}else{
							factor1 = +1;
							factor2 = 0;
						}
					break;
					case 1:
						if(mod==4){
							factor1 = -1;
							factor2 = +1;
						}else{
							factor1 = -1;
							factor2 = 0;
						}
					break;
					case 2:
						factor1 = +1;
						factor2 = -1;
					break;
					case 3:
						factor1 = -1;
						factor2 = -1;
					break;
				}
				break;
			}
		}
	}

// ---------------------------------------------------------------
	// 以下為 句子快取 的部份

	// TODO maybe we should extract cache operation to another class
	private static String CACHE_TABLE = "questionsentence";
	private static String[] CACHE_COL_NAMES = {"id", "questionsentence"};

	/**
	 * 從 快取id 取回 句子本身
	 */
	private static String getCacheString(int id) {
		SqlRowSet rs = DAO.select(String.format(
				"select %s from %s where %s=?;", CACHE_COL_NAMES[1],
				CACHE_TABLE, CACHE_COL_NAMES[0]), id);
		if(rs.next()){
			return rs.getString(1);
		}
		return "";
	}

	/**
	 * 取得此句子的 快取id。
	 * 若此句子從未出現在 快取 中，則自動新增。
	 */
	private static int getCacheId(String s) {
		// get cache id from database
		SqlRowSet rs = DAO.select(String.format(
				"select %s from %s where %s=?;", CACHE_COL_NAMES[0],
				CACHE_TABLE, CACHE_COL_NAMES[1]), (Object) s);

		if (rs.next()) {
			int ret = rs.getInt(1);
			return ret;
		}

		// if no, then insert one
		// TODO error checking on DAO.insert?
		int i = DAO.insert(CACHE_TABLE, CACHE_COL_NAMES, new Object[]{s});
		if (1 != i) {
			throw new RuntimeException("Question.getCacheId: DAO.insert");
		}
		return getCacheId(s);
	}

	/**
	 * 清除快取資料庫中未使用的問題句子
	 * 請注意呼叫時機，以避免 database lock
	 * TODO 如何自動化清理？
	 */
	public static void vacuum() {
//		System.out.println("cleaning up cache table!");
		ResultSet rs;
		ArrayList<Integer> cacheIds = new ArrayList<Integer>();
		// cache database clean up
		try {
			// for each string
			rs = DAO.select(String.format("select * from %s;", CACHE_TABLE));
			while (rs.next()) {
				cacheIds.add(rs.getInt(1));
			}
		} catch (SQLException e) {
			e.printStackTrace();
		} finally {
			DAO.close();
		}
			
			// search if any reference
		for (int id: cacheIds) {
			int count = -1;
			try {
//				System.out.println("exam id=" + id);
				rs = DAO.select(String.format("select count(*) from %s where %s='%s';", TABLE, COL_NAMES[2], id));
				rs.next();
				count = rs.getInt(1);
			} catch (SQLException e) {
				e.printStackTrace();
			} finally {
				DAO.close();
			}
			if (0 == count) {
				// if no => remove string
//				System.out.println("           delete!");
				DAO.delete(String.format("delete from %s where %s='%s';", CACHE_TABLE, CACHE_COL_NAMES[0], id));
			}
		}
	}
	
}
