package model;

import java.util.ArrayList;

import javax.swing.table.AbstractTableModel;

import tools.util.Constant;

// this TableModel is different from ExperimentsTableModel which is
// the real access entry of all the data.
// in contrary, this one is only a wrapper,
// all the questionnaires can be retrieved from the experiment which
// can be retrieved from ExperimentsTableModel
// as an result, this class can be created and abandoned arbitrarily.  

@SuppressWarnings("serial")
public class QuestionnairesTableModel extends AbstractTableModel {

	private static ArrayList<QuestionnairesTableModel> singletons = new ArrayList<QuestionnairesTableModel>();
	
	public static QuestionnairesTableModel getInstance(Experiment e) {
		for (QuestionnairesTableModel ret: singletons) {
			if (ret.getExperiment() == e) {
				return ret;
			}
		}
		
		QuestionnairesTableModel ret = new QuestionnairesTableModel(e);
		singletons.add(ret);
		return ret;
	}

	private Experiment experiment;

	private QuestionnairesTableModel(Experiment e) {
		experiment = e;
	}

	public Experiment getExperiment() {
		return experiment;
	}
	
	private static String[] COL_NAMES = {"no", "id", "notes", "state", "builddate", "experdate", "subject"};
	@Override
	public int getColumnCount() {
		return COL_NAMES.length;
	}

	private static String[] I18N_COL_NAMES = {Constant.NUMOF, Constant.ID, Constant.EXP_SURVEY_NOTES, Constant.EXP_SURVEY_STATE,
											Constant.EXP_SURVEY_BUILDDATE, Constant.EXP_SURVEY_EXPERDATE, Constant.EXP_SURVEY_SUBJECT};
	@Override
	public String getColumnName(int column) {
		return I18N_COL_NAMES[column];
	}

	@Override
	public int getRowCount() {
		return experiment.getQuestionnaires().size();
	}
	
	@Override
	public Object getValueAt(int arg0, int arg1) {
		if(arg1 == 0){
			return arg0+1;
		}
		
		Questionnaire q = experiment.getQuestionnaires().get(arg0);
		return getValueAt(q, arg1);
	}
	
	// for co-operating with TableModel.getValueAt
	private Object getValueAt(Questionnaire q, int colIdx) {
		if ("id".equals(COL_NAMES[colIdx])) {
			return q.getId();
		} else if ("notes".equals(COL_NAMES[colIdx])) {
			return q.getNotes();
		} else if ("state".equals(COL_NAMES[colIdx])) {
			return q.getState();
		} else if ("builddate".equals(COL_NAMES[colIdx])) {
			return q.getBuildDate();
		} else if ("experdate".equals(COL_NAMES[colIdx])) {
			return q.getAnswerDate();
		} else if ("subject".equals(COL_NAMES[colIdx])) {
			return q.getSubject();
		} else if ("questionexper_id".equals(COL_NAMES[colIdx])) {
			return experiment.getId();
		}
		return null;
	}

	public Object getId(int i){
		Questionnaire q = experiment.getQuestionnaires().get(i);
		return getValueAt(q, findColumn(Constant.ID));
	}
	
	public int findColumn(String columnName) {
		return super.findColumn(columnName);
	}
}
