package model;

import java.util.ArrayList;

import javax.swing.table.AbstractTableModel;

import tools.util.Constant;

@SuppressWarnings("serial")
public class QuestionsTableModel extends AbstractTableModel {

	private static ArrayList<QuestionsTableModel> singletons = new ArrayList<QuestionsTableModel>();
	
	public static QuestionsTableModel getInstance(Questionnaire q) {
		for (QuestionsTableModel ret: singletons) {
			if (ret.getQuestionnaire() == q) {
				return ret;
			}
		}
		
		QuestionsTableModel ret = new QuestionsTableModel(q);
		singletons.add(ret);
		return ret;
	}
	
	private QuestionsTableModel(Questionnaire q) {
		questionnaire = q;
	}
	
	// -------------------------------------------------------------
	
	private Questionnaire questionnaire;
	
	private Questionnaire getQuestionnaire() {
		return questionnaire;
	}
	
	// -------------------------------------------------------------
	
	private static String[] COL_NAMES = {"question", "isFake"};

	@Override
	public int getColumnCount() {
		return COL_NAMES.length;
	}

	private static String[] I18N_COL_NAMES = {Constant.EXP_QUE, Constant.EXP_SURSAMPLE_IS_FAKE};
	@Override
	public String getColumnName(int column) {
		return I18N_COL_NAMES[column];
	}

	@Override
	public int getRowCount() {
		return questionnaire.getQuestions().size();
	}

	@Override
	public Object getValueAt(int arg0, int arg1) {
		return getValueAt(questionnaire.getQuestions().get(arg0), arg1);
	}

	private Object getValueAt(Question q, int colIdx) {
		if ("id".equals(COL_NAMES[colIdx])) {
			return q.getId();
		} else if ("question".equals(COL_NAMES[colIdx])) {
			return q.getQuestion();
		} else if ("isFake".equals(COL_NAMES[colIdx])) {
			return q.isFake();
		}
		return null;
	}

}
