package model;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Random;
import java.util.UUID;

import junit.framework.TestCase;

import org.springframework.jdbc.support.rowset.SqlRowSet;

import tools.db.DAO;

public class TestExperiment extends TestCase {
	
	static int id = -1;
	
	private ResultSet rs;
	
	//------------建立實驗所需資料
	private static Experiment exp;
	private static int unique;
	private static String title;
	private static String positive;
	private static String negative;
	static int factorId = -1;
	static int fakeQuestionNum = 5;
	private static String guideTitle;
	private static String guideGuide;
	private int numOfQuestionnaire;
	
	private static Random rand = new Random();
	
	protected void setUp() throws Exception {
		super.setUp();
		createFactorData();
		
		//於測試之前建立一筆新的實驗
		numOfQuestionnaire = 0;		//因為只測實驗部分，所以問卷預設給0筆 
		exp = createQuestionExp(numOfQuestionnaire);
		
//		System.out.println("setUP ID(will be insert) : "+exp.getId());
	}

	static Experiment createQuestionExp(int numOfQuestionnaire) {
		unique = rand.nextInt(99999);
		title = "title" + unique;
		positive = "positive" + unique;
		negative = "negative" + unique;
		//雖然是由指導句模組來源的資料，但不產生關聯，所以可以由此assign值
		guideTitle = "guideTitle";
		guideGuide = "guideGuide";
		return (new Experiment(title, positive, negative, factorId, fakeQuestionNum, guideTitle, guideGuide, numOfQuestionnaire));
	}

	static void createFactorData() {
		String factor1 = UUID.randomUUID().toString().substring(0,4);
		String factor2 = UUID.randomUUID().toString().substring(0,4);
		String description = UUID.randomUUID().toString().substring(0,4);
		int total_sentences = 0;
		int i = DAO.insert(FactorsModel.TABLE, FactorsModel.COL_NAMES, new Object[]{factor1, factor2, description,total_sentences});
		assertEquals(1, i);
		try {
			ResultSet rs = DAO.select(String.format("select %s from %s where %s='%s'", FactorsModel.COL_NAMES[0], FactorsModel.TABLE, FactorsModel.COL_NAMES[3], description));
			factorId = rs.getInt(FactorsModel.COL_NAMES[0]);
			DAO.close();
		} catch (SQLException e) {
			e.printStackTrace();
		}
		String expTable = "experimental";
		String[] expCOL_NAMES = {"id", "title", "case1", "case2", "case3", "case4", "phrase_id"};
		String title = UUID.randomUUID().toString().substring(0,4);
		String case1 = UUID.randomUUID().toString().substring(0,4);
		String case2 = UUID.randomUUID().toString().substring(0,4);
		String case3 = UUID.randomUUID().toString().substring(0,4);
		String case4 = UUID.randomUUID().toString().substring(0,4);
		i = DAO.insert(expTable, expCOL_NAMES, new Object[]{title, case1, case2, case3, case4, factorId});
		assertEquals(1, i);
	}

	protected void tearDown() throws Exception {
		super.tearDown();
		//於測試結束之後將新增的實驗刪除
//		System.out.println("tearDown ID(will be Delete) : "+exp.getId());
//		exp.delete();
//		destoryFactorData();
	}
	
	static void destoryFactorData() {
		String expTable = "experimental";
		String[] expCOL_NAMES = {"id", "title", "case1", "case2", "case3", "case4", "phrase_id"};
		int i = DAO.delete(expTable, new String[]{expCOL_NAMES[6]}, new Object[]{factorId});
		assertEquals(1, i);
		i = DAO.delete(FactorsModel.TABLE, new String[]{FactorsModel.COL_NAMES[0]}, new Object[]{factorId});
		assertEquals(1, i);
		factorId = -1;
	}

	/**
	 * 測試getId()
	 */
	public void testGetId(){
		//假設如果新增實驗成功，則會回傳id, 失敗id = -1
		assertNotSame(-1, exp.getId());
	}
	
	/**
	 * 查看setUp的新增項目是否正確
	 */
	public void testExperiment(){
		try {
			String[] tmp = new String[7];
			//先檢查自己設的內容
			assertEquals(title, exp.getTitle());
			assertEquals(positive, exp.getPositive());
			assertEquals(negative, exp.getNegative());
			assertEquals(factorId, exp.getFactorId());
			assertEquals(fakeQuestionNum, exp.getFakeQuestionNum());
			assertEquals(guideTitle, exp.getGuideTitle());
			assertEquals(guideGuide, exp.getGuideGuide());
			assertEquals(numOfQuestionnaire, exp.getQuestionnaires().size());
			//再比對資料庫內的內容
			rs = DAO.select(String.format("select * from %s where %s='%s';", Experiment.TABLE, Experiment.COL_NAMES[0], exp.getId()));
			while(rs.next()){
				tmp[0] = rs.getString(Experiment.COL_NAMES[1]);
				tmp[1] = rs.getString(Experiment.COL_NAMES[2]);
				tmp[2] = rs.getString(Experiment.COL_NAMES[3]);
				tmp[3] = rs.getString(Experiment.COL_NAMES[4]);
				tmp[4] = rs.getString(Experiment.COL_NAMES[5]);
				tmp[5] = rs.getString(Experiment.COL_NAMES[7]);
				tmp[6] = rs.getString(Experiment.COL_NAMES[9]);
			}
			DAO.close();
			assertEquals(title, tmp[0]);
			assertEquals(positive, tmp[1]);
			assertEquals(negative, tmp[2]);
			assertEquals(factorId, Integer.parseInt(tmp[3]));
			assertEquals(fakeQuestionNum+"", tmp[4]);
			assertEquals(guideTitle, tmp[5]);
			assertEquals(guideGuide, tmp[6]);			
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * 由於setUp有建立實驗，而tearDown刪除了實驗
	 * 所以在此測試delete是否能正常運作
	 */
	public void testDelete(){
		try {
			//先檢查實驗是否存在於Experiment的資料裡面(是)
			ArrayList<Experiment> e = ExperimentsTableModel.getInstance().getAllExperiments();
			boolean b = false;
			for(int j=0;j<e.size();j++){
				if(e.get(j).getId() == exp.getId()){
					b = true;
					break;
				}
			}
			assertTrue(b);
			
			//再檢查實驗是否存在於資料庫裡面(是)
			rs = DAO.select(String.format("select count(*) from %s where %s='%s'", Experiment.TABLE, Experiment.COL_NAMES[0], exp.getId()));
			int i = rs.getInt(1);
			DAO.close();
			assertEquals(1, i);
			
			//刪除資料
			exp.delete();
			//先檢查實驗是否存在於Experiment的資料裡面(否)
			e = Experiment.restoreAllExperiments();
			b = false;
			for(int j=0;j<e.size();j++){
				if(e.get(j).getId() == exp.getId()){
					b = true;
					break;
				}
			}
			assertFalse(b);
			//再檢查實驗是否存在於資料庫裡面(否)
			rs = DAO.select(String.format("select count(*) from %s where %s='%s'", Experiment.TABLE, Experiment.COL_NAMES[0], exp.getId()));
			i = rs.getInt(1);
			DAO.close();
			assertEquals(0, i);
			
			//!!由實驗刪除問卷部分由問卷的地方實作(因預設問卷數為0)
			//建立實驗供tearDown刪除
			exp = new Experiment(title, positive, negative, factorId, fakeQuestionNum, guideTitle, guideGuide, numOfQuestionnaire);
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * 測試restoreAllExperiments()
	 */
	public void testRestoreAllExperiments(){
		//由資料庫取回所有的實驗
		ArrayList<ArrayList<String>> rae = new ArrayList<ArrayList<String>>();
		try {
			//此部分會檢查到降冪
			rs = DAO.select(String.format("select * from %s order by %s desc;", Experiment.TABLE, Experiment.COL_NAMES[0]));
			
			while (rs.next()) {
				ArrayList<String> e = new ArrayList<String>();
				e.add(rs.getString(Experiment.COL_NAMES[1]));
				e.add(rs.getString(Experiment.COL_NAMES[2]));
				e.add(rs.getString(Experiment.COL_NAMES[3]));
				e.add(rs.getString(Experiment.COL_NAMES[4]));
				e.add(rs.getString(Experiment.COL_NAMES[5]));
				e.add(rs.getString(Experiment.COL_NAMES[7]));
				e.add(rs.getString(Experiment.COL_NAMES[9]));
				rae.add(e);
			}
		} catch (SQLException e) {
			e.printStackTrace();
		} finally {
			DAO.close();
		}
		//比對取得的項目數目是否相同
		assertEquals(rae.size(), Experiment.restoreAllExperiments().size());
		//比對資料
		int size = rae.size();
		for(int i=0;i<size;i++){
			assertEquals(rae.get(i).get(0), Experiment.restoreAllExperiments().get(i).getTitle());	
//			assertEquals(rae.get(i).get(1), Experiment.restoreAllExperiments().get(i).getPositive());
//			assertEquals(rae.get(i).get(2), Experiment.restoreAllExperiments().get(i).getNegative());
//			assertEquals(rae.get(i).get(3), Experiment.restoreAllExperiments().get(i).getFactorDescription());
//			assertEquals(rae.get(i).get(4), Experiment.restoreAllExperiments().get(i).getFakeQuestionNum()+"");
//			assertEquals(rae.get(i).get(5), Experiment.restoreAllExperiments().get(i).getGuideTitle());
//			assertEquals(rae.get(i).get(6), Experiment.restoreAllExperiments().get(i).getGuideGuide());
			//測太多會跑很久
		}		
	}
	
	/**
	 * 測試setTitle()
	 */
	public void testSetTitle(){
		try {
			String title = UUID.randomUUID().toString().substring(0, 3);
			exp.setTitle(title);
			//檢查title是否與設的title相同
			assertEquals(title, exp.getTitle());
			//檢查資料庫的title是否與設的title相同
			rs = DAO.select(String.format("select %s from %s where %s='%s';", Experiment.COL_NAMES[1], Experiment.TABLE, Experiment.COL_NAMES[0], exp.getId()));
			String s = rs.getString(1);
			DAO.close();
			assertEquals(title, s);
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}
	
	public void testt(){
		try {
			rs = DAO.select(String.format("select %s from %s where %s='%s';", Experiment.COL_NAMES[1], Experiment.TABLE, Experiment.COL_NAMES[0], exp.getId()));
			String s;
			s = rs.getString(1);
			DAO.close();
			System.out.println("s : "+s);
		} catch (SQLException e) {
			e.printStackTrace();
		}
//		Map<String, Object> result = DAO.getJdbcTemplate().queryForMap("select ? from ? where ?=?;", new Object[]{Experiment.COL_NAMES[1], Experiment.TABLE, Experiment.COL_NAMES[0], exp.getId()});
//		while(result.keySet().iterator().hasNext()
		SqlRowSet sql = DAO.select("select * from " + Experiment.TABLE + " where id=?;", exp.getId());
		while(sql.next()){
			System.out.println(sql.getString(Experiment.COL_NAMES[1]));
		}
	}
}
