package model;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.UUID;

import junit.framework.TestCase;
import tools.db.DAO;

public class TestFakeQuestionsModel extends TestCase {
	
	// TODO you should use the constants in FakeQuestionsModel
	private static final String TABLE = "interference";
	private static final String[] COL_NAMES = {"id", "interference"};
	private FakeQuestionsModel fqm;
	private String uuid1;
	private ResultSet rs;
	
	protected void setUp() throws Exception {
		super.setUp();
		fqm = FakeQuestionsModel.getTableModel();
		uuid1 = UUID.randomUUID().toString().substring(0, 4);
	}

	protected void tearDown() throws Exception {
		super.tearDown();
	}
	
	public void testGetRowCount(){
		try {
			rs = DAO.select(String.format("select count(%s) from %s", COL_NAMES[1], TABLE));
			assertEquals(rs.getInt(1), fqm.getRowCount());
		} catch (SQLException e) {
			e.printStackTrace();
		} finally {
			DAO.close();
		}
	}
	
	public void testGetValueAt(){
		
		try {
			ArrayList<ArrayList<String>> allData = new ArrayList<ArrayList<String>>();
			ArrayList<String> rowData = null;
			rs = DAO.select(String.format("select * from %s order by %s desc", TABLE, COL_NAMES[0]));
			while(rs.next()){
				rowData = new ArrayList<String>();
				for(int i=0;i<COL_NAMES.length;i++){
					rowData.add(rs.getString(COL_NAMES[i]));
				}
				allData.add(rowData);
			}
			for(int i=0;i<allData.size();i++){
				for(int j=0;j<COL_NAMES.length;j++){
					assertEquals(allData.get(i).get(j), fqm.getValueAt(i, j));
				}
			}
		} catch (SQLException e) {
			e.printStackTrace();
		} finally {
			DAO.close();
		}
	}
	
	public void testGetId(){
		try {
			rs = DAO.select(String.format("select * from %s order by id desc", TABLE));
			rs.next();
			assertEquals(rs.getString(COL_NAMES[0]), FakeQuestionsModel.getId(rs.getString(COL_NAMES[1]))+"");
		} catch (SQLException e) {
			e.printStackTrace();
		} finally {
			DAO.close();
		}
	}
	
	public void testGetAllQuestions(){
		ArrayList<String> ls = new ArrayList<String>();
		try {
			rs = DAO.select(String.format("select %s from %s order by %s desc;", COL_NAMES[1], TABLE, COL_NAMES[0]));
			while(rs.next()){
				ls.add(rs.getString(COL_NAMES[1]));
			}
			assertEquals(ls, FakeQuestionsModel.getAllQuestions());
		} catch (SQLException e) {
			e.printStackTrace();
		} finally {
			DAO.close();
		}
	}
	
	public void testUpdate(){
		try {
			String tmpId,tmpQuestion1,tmpQuestion2;
			rs = DAO.select(String.format("select * from %s order by id desc", TABLE));
			tmpId = rs.getString(COL_NAMES[0]);
			tmpQuestion1 = rs.getString(COL_NAMES[1]);
			DAO.close();
			//試做更新動作
			assertEquals(true, FakeQuestionsModel.update(Integer.parseInt(tmpId), uuid1));
			
			rs = DAO.select(String.format("select * from %s order by id desc", TABLE));
			tmpQuestion2 = rs.getString(COL_NAMES[1]);
			DAO.close();
			//檢查更新資料
			assertEquals(uuid1, tmpQuestion2);
			
			FakeQuestionsModel.update(Integer.parseInt(tmpId), tmpQuestion1);
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}
	
	public void testInsertDelete(){
		try {
			int oId,newId,num,numAfterDelete;
			rs = DAO.select(String.format("select * from %s order by id  desc", TABLE));
			oId = rs.getInt(COL_NAMES[0]);
			DAO.close();
			//新增的id會等於最大的id+1
			assertEquals(oId+1, FakeQuestionsModel.insert(uuid1));
			
			rs = DAO.select(String.format("select * from %s where id = %s order by id desc", TABLE, oId+1));
			//確定新增是否正常
			assertEquals(uuid1, rs.getString(COL_NAMES[1]));
			DAO.close();
			
			rs = DAO.select(String.format("select %s from %s order by id  desc",COL_NAMES[0] ,TABLE));
			newId = rs.getInt(1);
			DAO.close();
			rs = DAO.select(String.format("select count(*) from %s order by id  desc", TABLE));
			num = rs.getInt(1);
			DAO.close();
			//測試刪除功能
			assertEquals(true, FakeQuestionsModel.delete(newId));
			
			rs = DAO.select(String.format("select count(*) from %s order by id  desc", TABLE));
			numAfterDelete = rs.getInt(1);
			DAO.close();
			//刪除後數量
			assertEquals(num-1, numAfterDelete);
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}
	
	public static void main(String[] args){
		junit.swingui.TestRunner.run(FakeQuestionsModel.class);
	}
}
