package model;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.UUID;

import junit.framework.TestCase;
import tools.db.DAO;

public class TestGuidesModel extends TestCase {
	
	private GuidesModel gm;
	private ResultSet rs;
	private String title;
	private String guide;
	
	protected void setUp() throws Exception {
		super.setUp();
		rs = null;
		gm = GuidesModel.getTableModel();
		//建立將測試的資料
		title = UUID.randomUUID().toString().substring(0, 4);
		guide = UUID.randomUUID().toString().substring(0, 4);
		//  ' 和 " 字元測試在 testNewXXXXMethod有測試
	}

	protected void tearDown() throws Exception {
		super.tearDown();
		rs = null;
		gm = null;
		title = null;
		guide = null;
	}
	
	/**
	 * 測試getRowCount()
	 */
	public void testGetRowCount(){
		try {
			ResultSet rs = DAO.select("select count(*) from "+ GuidesModel.TABLE);
			int i = rs.getInt(1);
			DAO.close();
			//從資料庫取得筆數後比對
			assertEquals(gm.getRowCount(), i);
		} catch (SQLException e) {
			e.printStackTrace();			
		}
	}
	
	/**
	 * 測試getValueAt()
	 */
	public void testGetValueAt(){
		Object obj = null;
		try {
			for(int i=0;i<gm.getRowCount();i++){
				for(int j=0;j<gm.getColumnCount();j++){
					rs = DAO.select(String.format("select %s from %s order by %s desc", GuidesModel.COL_NAMES[j], GuidesModel.TABLE, GuidesModel.COL_NAMES[0]));
					int x = i;
					while(rs.next()){
						if(-1 == --x){
							obj = rs.getString(1);
							break;
						}
					}
					DAO.close();
					//如果非降冪的資料將會false
					//比對由資料庫取得與getValueAt的method結果是否相同
					assertEquals(gm.getValueAt(i, j), obj);
				}
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * 測試新增資料(insert())
	 */
	public void testInsert(){
		try {
			int rows = gm.getRowCount();
			//測試新增一筆資料
			int id = GuidesModel.insert(title, guide);
			//回傳值應該要成功
			assertTrue(-1 != id);
			
			rs = DAO.select(String.format("select count(*) from %s;", GuidesModel.TABLE));
			rs.next();
			int rowsAfterInsert = rs.getInt(1);
			DAO.close();
			//檢查筆數是否增加一筆
			assertEquals(rows+1, rowsAfterInsert);
			
			//取回最新一筆資料並比對
			rs = DAO.select(String.format("select * from %s order by id desc;", GuidesModel.TABLE));
			rs.next();
			String col1 = rs.getString(GuidesModel.COL_NAMES[1]);
			String col2 = rs.getString(GuidesModel.COL_NAMES[2]);
			DAO.close();
			assertEquals(col1, title);
			assertEquals(col2, guide);
			
			//測試新增一筆重覆的資料
			id = GuidesModel.insert(title, guide);
			//回傳值應該要失敗
			assertTrue(-1 == id);
			
			//確認資料庫該筆資料只有一筆
			rs = DAO.select(String.format("select count(*) from %s where %s='%s' and %s='%s';", GuidesModel.TABLE, GuidesModel.COL_NAMES[1], title, GuidesModel.COL_NAMES[2], guide));
			rs.next();
			int num = rs.getInt(1);
			DAO.close();
			assertEquals(1, num);
			
			//刪資測試的資料
			int i = DAO.delete(String.format("delete from %s where %s='%s' and %s='%s'", GuidesModel.TABLE, GuidesModel.COL_NAMES[1], title, GuidesModel.COL_NAMES[2], guide));
			assertEquals(1, i);
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * 測試刪除資料(delete())
	 */
	public void testDelete(){
		try {
			String tmpTitle,tmpGuide;
			//新增一筆資料
			int id = GuidesModel.insert(title, guide);
			//取得該筆資料
			rs = DAO.select(String.format("select * from %s where %s='%s';", GuidesModel.TABLE, GuidesModel.COL_NAMES[0], id));
			tmpTitle = rs.getString(GuidesModel.COL_NAMES[1]);
			tmpGuide = rs.getString(GuidesModel.COL_NAMES[2]);
			DAO.close();
			//刪除該筆資料
			boolean b = GuidesModel.delete(id);
			//確認可刪除
			assertTrue(b);
			
			//利用新增的資料去搜尋資料庫
			rs = DAO.select(String.format("select count(*) from %s where %s='%s' and %s='%s';", GuidesModel.TABLE, GuidesModel.COL_NAMES[1], tmpTitle, GuidesModel.COL_NAMES[2], tmpGuide));
			//結果應不存在
			int num = rs.getInt(1);
			DAO.close();
			assertEquals(0, num);
			
			//測試刪除不存在的資料
			b = GuidesModel.delete(id);
			assertFalse(b);
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * 測試編輯資料(update())
	 */
	public void testUpdate(){
		try {
			//取得將更新資料的內容(以最新一筆為例子)
			String tmpId,tmpTitle1,tmpGuide1,tmpTitle2,tmpGuide2;
			rs = DAO.select(String.format("select * from %s order by id desc", GuidesModel.TABLE));
			rs.next();
			//暫存該筆資料
			tmpId = rs.getString(GuidesModel.COL_NAMES[0]);
			tmpTitle1 = rs.getString(GuidesModel.COL_NAMES[1]);
			tmpGuide1 = rs.getString(GuidesModel.COL_NAMES[2]);
			DAO.close();
			//上傳更新該筆資料，應為正常可更新
			assertEquals(true, GuidesModel.update(Integer.parseInt(tmpId), title, guide));
			
			//取得該筆更新的資料
			rs = DAO.select(String.format("select * from %s order by id desc", GuidesModel.TABLE));
			rs.next();
			tmpTitle2 = rs.getString(GuidesModel.COL_NAMES[1]);
			tmpGuide2 = rs.getString(GuidesModel.COL_NAMES[2]);
			DAO.close();
			//檢查更新後的東西是否正確
			assertEquals(title,tmpTitle2);
			assertEquals(guide,tmpGuide2);
			
			//測試更新不存在的資料(最新一筆的之後再十筆)
			assertEquals(false, GuidesModel.update(Integer.parseInt(tmpId+10), title, guide));
			
			//通過測試後，將原本的資料寫回
			GuidesModel.update(Integer.parseInt(tmpId), tmpTitle1, tmpGuide1);
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * 測試getTitle() & getGuide()
	 */
	public void testGetTitleAndGuide(){
		//新增一筆資料
		int id = GuidesModel.insert(title, guide);
		//取得內容後比對
		assertEquals(title, GuidesModel.getTitle(id));
		assertEquals(guide, GuidesModel.getGuide(id));
		//刪除新增得資料
		GuidesModel.delete(id);
	}
	
	/**
	 * by Spring
	 */
//	public void testNewInsertDeleteMethod(){
//		title += "'";
//		guide += "\"";
//		int i = DAO.insert(GuidesModel.TABLE, GuidesModel.COL_NAMES, new Object[]{title, guide});
//		assertEquals(1, i);
//		i = DAO.delete(GuidesModel.TABLE, new String[]{GuidesModel.COL_NAMES[1],GuidesModel.COL_NAMES[2]}, new Object[]{title, guide});
//		assertEquals(1, i);
//	}
}
