package model;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Random;
import java.util.UUID;

import org.springframework.jdbc.support.rowset.SqlRowSet;

import tools.db.DAO;

import junit.framework.TestCase;

public class TestQuestion extends TestCase {

	private ResultSet rs;
	private SqlRowSet sqlRs;
	private Experiment exp;
	
	private static String CACHE_TABLE = "questionsentence";
	private static String[] CACHE_COL_NAMES = {"id", "questionsentence"};
	
	private ArrayList<Questionnaire> que;
	private Questionnaire questionnaire;
	private ArrayList<Question> questions;

	private final int numOfQuestionnaire = 1;
	
	private Random rand = new Random();
	
	protected void setUp() throws Exception {
		super.setUp();
		TestExperiment.createFactorData();
		
		//於測試之前建立一筆新的實驗
		//問卷的產生會依存一個實驗，所以在這先建立一個實驗
		exp = TestExperiment.createQuestionExp(numOfQuestionnaire);
		que = exp.getQuestionnaires();
		//取得該實驗的其中一份問卷
		questionnaire = que.get(rand.nextInt(que.size()));
		//再由此份問卷內的問題來做測試
		questions = questionnaire.getQuestions();
	}

	protected void tearDown() throws Exception {
		super.tearDown();
		//將實驗刪除時會連帶的將問卷刪除
		exp.delete();
		que = null;
		questionnaire = null;
		TestExperiment.destoryFactorData();
	}
	
	/**
	 * 測試getId()
	 */
	public void testGetId(){
		//假設如果新增問題成功，則會回傳id, 失敗id = -1
		for(int i=0;i<questions.size();i++){
			assertNotSame(-1, questions.get(i).getId());
		}
	}
	
	/**
	 * 驗證新增問題
	 */
	public void testQuestion(){
		try {
			int numOfQuestion = questions.size();
			String sentence = UUID.randomUUID().toString().substring(0, 4);
			boolean isFake = rand.nextBoolean();
			//測試新增了問題之後是否存依在該問卷底下
			Question q = new Question(sentence, isFake, questionnaire);
			assertEquals(questionnaire, q.getQuestionnaire());
			//該問卷底下的問題會多 1
			assertEquals(numOfQuestion+1, questionnaire.getQuestions().size());
			
			//其內容是否符合預設的設定(先實體，後資料庫)
			//句子
			assertEquals(sentence, q.getQuestion());
			rs = DAO.select(String.format("select %s from %s where %s='%s'", 
					Question.COL_NAMES[2], Question.TABLE, Question.COL_NAMES[0], q.getId()));
			String questionId = rs.getString(1);
			DAO.close();
			rs = DAO.select(String.format("select %s from %s where %s='%s'", 
					CACHE_COL_NAMES[1], CACHE_TABLE, CACHE_COL_NAMES[0], questionId));
			String s = rs.getString(1);
			DAO.close();
			assertEquals(sentence, s);
			//是否為干擾句
			assertEquals(isFake, q.isFake());
			rs = DAO.select(String.format("select %s from %s where %s='%s'", 
					Question.COL_NAMES[3], Question.TABLE, Question.COL_NAMES[0], q.getId()));
			boolean b = Boolean.parseBoolean(rs.getString(1));
			DAO.close();
			assertEquals(isFake, b);
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * 測試問題刪除功能
	 */
	public void testDelete(){
		try {
//			questions = Question.restoreQuestions(questionnaire);
			Question q =  questions.get(rand.nextInt(questions.size()));
			//先檢查問題是否存在於問卷的資料裡面(是)
			boolean b = false;
			for(int j=0;j<questions.size();j++){
				if(questions.get(j).getId() == q.getId()){
					b = true;
					break;
				}
			}
			assertTrue(b);
			
			//再檢查問題是否存在於資料庫裡面(是)
			rs = DAO.select(String.format("select count(*) from %s where %s='%s'",
					Question.TABLE, Question.COL_NAMES[0], q.getId()));
			int i;
			i = rs.getInt(1);
			DAO.close();
			assertEquals(1, i);
			
			//刪除資料
			q.delete();
			//先檢查問題是否存在於問卷的資料裡面(否)
//			questions = Question.restoreQuestions(questionnaire);
			b = false;
			for(int j=0;j<questions.size();j++){
				if(questions.get(j).getId() == q.getId()){
					b = true;
					break;
				}
			}
			assertFalse(b);
			
			//再檢查問題是否存在於資料庫裡面(否)
			rs = DAO.select(String.format("select count(*) from %s where %s='%s'",
					Question.TABLE, Question.COL_NAMES[0], q.getId()));
			i = rs.getInt(1);
			DAO.close();
			assertEquals(0, i);
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * 驗證目前所新增的問題內容是否正確
	 */
	public void testRestoreQuestions(){
		//questions為questionnaire裡面restoreQuestions()而來，故驗證此變數
		//由問卷取得問題的數量應與restoreQuestions()而來的問題數量相同
		assertEquals(questionnaire.getQuestions().size(), questions.size());
		//驗證問題是否存在於該問卷中
		for(int i=0;i<questions.size();i++){
			checkContent(i);
		}
	}
	
	private void checkContent(int i) {
		String[] tmp = new String[4];
		String sql = "select * from " + Question.TABLE + " where id = ?;";
		
		sqlRs = DAO.select(sql, questions.get(i).getId());
		
		if(sqlRs.next()){
			tmp[0] = sqlRs.getString(Question.COL_NAMES[1]);
			tmp[1] = sqlRs.getString(Question.COL_NAMES[2]);
			tmp[2] = sqlRs.getString(Question.COL_NAMES[3]);
			tmp[3] = sqlRs.getString(Question.COL_NAMES[4]);
		}
		
		sql = "select * from " + CACHE_TABLE + " where id = ?;";
		sqlRs = DAO.select(sql, new Object[]{tmp[1]});
		
		if(sqlRs.next()){
			tmp[1] = sqlRs.getString(CACHE_COL_NAMES[1]);
		}
		
		assertEquals(tmp[0], questions.get(i).getQuestionnaire().getId()+"");
		assertEquals(tmp[1], questions.get(i).getQuestion());
		assertEquals(tmp[2], questions.get(i).isFake()+"");
		assertEquals(tmp[3], questions.get(i).getAnswer()+"");
	}
	
	/**
	 * 測試set
	 */
	public void testSet(){
		//隨機取一個問題
		Question q =  questions.get(rand.nextInt(questions.size()));
		//設定set值
		boolean answer = rand.nextBoolean(),tmpAnswer;
		q.setAnswer(answer);
		//檢查get值是否相同
		assertEquals(answer, q.getAnswer());
		try {
			//檢查資料庫是否有寫入
			rs = DAO.select(String.format("select * from %s where %s='%s';", 
					Question.TABLE, Question.COL_NAMES[0], q.getId()));
			tmpAnswer = Boolean.parseBoolean(rs.getString(Question.COL_NAMES[4]));
			DAO.close();
			assertEquals(answer, tmpAnswer);
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}
}
