package model;

import java.sql.Date;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Random;

import tools.db.DAO;

import junit.framework.TestCase;

public class TestQuestionnaire extends TestCase {

	private ResultSet rs;
	private Experiment exp;
	private ArrayList<Questionnaire> que;

	private int numOfQuestionnaire = 2;	
	//在此給定實驗將會產生的問卷數
	//問卷數最好>=2，方便delete刪除
	
	private Random rand = new Random();
	
	protected void setUp() throws Exception {
		super.setUp();
		TestExperiment.createFactorData();
		
		//於測試之前建立一筆新的實驗
		numOfQuestionnaire = 2 + rand.nextInt(3);	//產生2~4份問卷
		
		//問卷的產生會依存一個實驗，所以在這先建立一個實驗
		exp = TestExperiment.createQuestionExp(numOfQuestionnaire);
		//取得該實驗的問卷來做測試
		que = exp.getQuestionnaires();
	}

	protected void tearDown() throws Exception {
		super.tearDown();
		//將實驗刪除時會連帶的將問卷刪除
		exp.delete();
		que = null;
		TestExperiment.destoryFactorData();
	}
	
	/**
	 * 測試getId()
	 */
	public void testGetId(){
		//假設如果新增問卷成功，則會回傳id, 失敗id = -1
		for(int i=0;i<que.size();i++){
			assertNotSame(-1, que.get(i).getId());
		}
	}
	
	/**
	 * 驗證新增問卷
	 */
	public void testQuestionnaire(){
		try {
			int numOfQuestionnaire = exp.getQuestionnaires().size();
			//測試新增問卷之後是否存依在該實驗底下
			Questionnaire q = new Questionnaire(exp);
			assertEquals(exp, q.getExperiment());
			//該實驗底下的問卷數多 1
			assertEquals(numOfQuestionnaire+1, exp.getQuestionnaires().size());
			
			//其內容是否符合預設的設定(先實體，後資料庫)
			//狀態
			assertEquals(Questionnaire.State.NEW, q.getState());
			rs = DAO.select(String.format("select %s from %s where %s='%s'", 
					Questionnaire.COL_NAMES[2], Questionnaire.TABLE, Questionnaire.COL_NAMES[0], q.getId()));
			String state = rs.getString(1);
			rs.close();
			assertEquals(Questionnaire.State.NEW, Questionnaire.State.valueOf(state));
			
			//建立日期
			String today = (new Date(Calendar.getInstance().getTimeInMillis()))+"";
			assertEquals(today, q.getBuildDate()+"");
			rs = DAO.select(String.format("select %s from %s where %s='%s'", 
					Questionnaire.COL_NAMES[3], Questionnaire.TABLE, Questionnaire.COL_NAMES[0], q.getId()));
			String date = rs.getDate(1).toString();
			rs.close();
			assertEquals(today, date);
			
			//干擾句數目是否正確
			int numOfAllSentence = q.getQuestions().size();
			int numOfIsFake = 0;
			for(int i=0;i<numOfAllSentence;i++){
				if(q.getQuestions().get(i).isFake()){
					numOfIsFake++;
				}
			}
			assertEquals(TestExperiment.fakeQuestionNum, numOfIsFake);
			
			//實驗句庫的句子數目是否正確
//			rs = DAO.select(String.format("select %s from %s where %s='%s';", 
//					"id", "phrase", "description", factorId));
//			int phraseId = rs.getInt(1);
//			rs.close();
			rs = DAO.select(String.format("select count(*) from %s where %s='%s';", 
					"experimental", "phrase_id", TestExperiment.factorId));
			int numOfExpSentence = rs.getInt(1)*4;
			rs.close();
			assertEquals(numOfExpSentence, numOfAllSentence - numOfIsFake);
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * 由於setUp有建立實驗，建立實驗時也會建立問卷，而tearDown刪除了實驗
	 * 所以在此測試delete是否能正常運作
	 * 
	 * 因為刪除了問卷會連帶的刪除問題，因為多測試了刪除問卷的時候，連帶刪除問題的結果
	 * 因為在實驗的刪除部分會連帶的刪除問卷，因此多測試刪除實驗的時候，連帶刪除問卷的結果
	 * 
	 * Henry:
	 * 在 Questionnaire 測試 delete 的方法/目的應該是：
	 * 1. 測試delete 之後會不會正確的反應於 experiment 裡頭的 questionnaires 跟 資料庫上
	 * 2. 並且其下的問題也一併被刪除
	 */
	public void testDelete(){
		try {
			//檢查問卷是否>=2 (若小於1看不太出結果)
			if(que.size() < 2){
				for(int j=que.size();j<2;j++){
					new Questionnaire(exp);
					System.out.println("new Questionnaire(exp)");
				}
			}
//			que = Questionnaire.restoreQuestionnaires(exp);
			Questionnaire questionnaire = que.get(rand.nextInt(que.size()));
			//先檢查問卷是否存在於Questionnaire的資料裡面(是)
			boolean b = false;
			for(int j=0;j<que.size();j++){
				if(que.get(j).getId() == questionnaire.getId()){
					b = true;
					break;
				}
			}
			assertTrue(b);
			
			//再檢查問卷是否存在於資料庫裡面(是)
			rs = DAO.select(String.format("select count(*) from %s where %s='%s'",
					Questionnaire.TABLE, Questionnaire.COL_NAMES[0], questionnaire.getId()));
			int i = rs.getInt(1);
			DAO.close();
			assertEquals(1, i);
			
			//刪除前先暫存問卷的問題，用來檢查是否會連動刪除問題
			ArrayList<Question> tmpQ = questionnaire.getQuestions();
			//刪除問卷
			questionnaire.delete();
			//先檢查問卷是否存在於Questionnaire的資料裡面(否)
//			que = Questionnaire.restoreQuestionnaires(exp);
			b = false;
			for(int j=0;j<que.size();j++){
				if(que.get(j).getId() == questionnaire.getId()){
					b = true;
					break;
				}
			}
			assertFalse(b);
			//再檢查問卷是否存在於資料庫裡面(否)
			rs = DAO.select(String.format("select count(*) from %s where %s='%s'", Questionnaire.TABLE, Questionnaire.COL_NAMES[0], questionnaire.getId()));
			i = rs.getInt(1);
			DAO.close();
			assertEquals(0, i);
			//-------------------與問題連動-------------------
			//最後再檢查問題是否還在資料庫裡面(否)
			for(int j=0;j<tmpQ.size();j++){
				rs = DAO.select(String.format("select count(*) from %s where %s='%s'", Question.TABLE, Question.COL_NAMES[0], tmpQ.get(j).getId()));
				i = rs.getInt(1);
				DAO.close();
				assertEquals(0, i);
			}
			
			//-------------------與實驗連動部分-------------------
			//取得實驗內所有的問卷
//			que = Questionnaire.restoreQuestionnaires(exp);
			//刪除實驗
			exp.delete();
			//是否有與該實驗關聯的問卷存在於資料庫裡面(否)
			for(int j=0;j<que.size();j++){
				rs = DAO.select(String.format("select count(*) from %s where %s='%s'", Questionnaire.TABLE, Questionnaire.COL_NAMES[0], que.get(j).getId()));
				i = rs.getInt(1);
				DAO.close();
				assertEquals(0, i);
			}
			
			//建立實驗供tearDown刪除
			exp = TestExperiment.createQuestionExp(numOfQuestionnaire);
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * 測試新增的問卷的內容是否正確
	 * restoreQuestionnaires()
	 */
	public void testRestoreQuestionnaires(){
		//問卷的數應等於建立實驗時所給的問卷數
		assertEquals(numOfQuestionnaire, que.size());
		//檢查每份問卷的內容是否正確
		for(int i=0;i<numOfQuestionnaire;i++){
			checkContent(i);
		}
	}
	
	private void checkContent(int i){
		try {
			//que裡面的資料就是Questionnaire.restoreQuestionnaires(exp);
			//比對取出來的資料與資料庫是否相同
			String[] tmp = new String[6];
			rs = DAO.select(String.format("select * from %s where %s='%s'", 
					Questionnaire.TABLE, Questionnaire.COL_NAMES[0], que.get(i).getId()));
			tmp[0] = rs.getString(Questionnaire.COL_NAMES[1]);
			tmp[1] = rs.getString(Questionnaire.COL_NAMES[2]);
			tmp[2] = (null == rs.getDate(Questionnaire.COL_NAMES[3])) ? null : rs.getDate(Questionnaire.COL_NAMES[3]).toString();
			tmp[3] = (null == rs.getDate(Questionnaire.COL_NAMES[4])) ? null : rs.getDate(Questionnaire.COL_NAMES[4]).toString();
			tmp[4] = rs.getString(Questionnaire.COL_NAMES[5]);
			tmp[5] = rs.getString(Questionnaire.COL_NAMES[6]);
			DAO.close();
			assertEquals(tmp[0], que.get(i).getNotes());
			assertEquals(tmp[1], que.get(i).getState()+"");
			assertEquals(tmp[2], que.get(i).getBuildDate()+"");
			assertEquals(tmp[3], que.get(i).getAnswerDate());
			assertEquals(tmp[4], que.get(i).getSubject());
			assertEquals(tmp[5], exp.getId()+"");
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * 測試setSubject, setNotes, setState, setAnswerDate
	 */
	public void testSet(){
		Questionnaire q = que.get(rand.nextInt(que.size()));
		//設定set值
		String notes = "message", tmpNotes;
		String subject = "XXX", tmpSubject;
		String tmpState,tmpDate;
		Questionnaire.State state = Questionnaire.State.COMPLETED;
		Date date = new Date(Calendar.getInstance().getTimeInMillis());
		q.setNotes(notes);
		q.setSubject(subject);
		q.setState(state);
		q.setAnswerDate(date);
		//檢查get值是否相同
		assertEquals(notes, q.getNotes());
		assertEquals(subject, q.getSubject());
		assertEquals(state, q.getState());
		assertEquals(date, q.getAnswerDate());
		try {
			//檢查資料庫是否有寫入
			rs = DAO.select(String.format("select * from %s where %s='%s'", 
					Questionnaire.TABLE, Questionnaire.COL_NAMES[0], q.getId()));
			tmpNotes = rs.getString(Questionnaire.COL_NAMES[1]);
			tmpState = rs.getString(Questionnaire.COL_NAMES[2]);
			tmpDate = rs.getDate(Questionnaire.COL_NAMES[4]).toString();
			tmpSubject = rs.getString(Questionnaire.COL_NAMES[5]);
			DAO.close();
			assertEquals(notes, tmpNotes);
			assertEquals(subject, tmpSubject);
			assertEquals(state+"", tmpState);
			assertEquals(date+"", tmpDate);
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}
}
