package tools.db;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;

import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.jdbc.support.rowset.SqlRowSet;

import vo.PhraseTable;
/**
 * 處理對資料庫要求 CRUD 等行為
 * @author 阿助
 *
 */
public class DAO {
	private static ArrayList<Statement> stList = new ArrayList<Statement>();
	
	public static PhraseTable preSelect(String sql, Object[] object, RowMapper<PhraseTable> rowMapper){
		JdbcTemplate jdbcTemplate = DBConnector.getJdbcTemplate();
		return jdbcTemplate.queryForObject(sql, object, rowMapper);
	}
	
	public static JdbcTemplate getJdbcTemplate(){
		return DBConnector.getJdbcTemplate();
	}
	
	public static int jdbcUpdate(String sql, Object[] object){
		JdbcTemplate jdbcTemplate = DBConnector.getJdbcTemplate();
		return jdbcTemplate.update(sql, object);
	}
	
	/**
	 * <br>執行 select 。
	 * @param sql sql 語法。
	 * @return ResultSet select 的結果。
	 */
	public static ResultSet select(String sql){
		Statement st = DBConnector.getStatement();
		stList.add(st);
		try {
			return st.executeQuery(sql);
		} catch (SQLException e) {
			e.printStackTrace();
			return null;
		}
	}
	
	/**
	 * <br>執行 select 。可依照需求輸入所需的 where 條件。
	 * @param tableName 資料表名稱。
	 * @param conditions select 所需的 where 條件。
	 * @return ResultSet select 的結果。
	 */
	public static ResultSet select(String tableName, String conditions){
		String sql = "select * from " + tableName + conditions + ";";
		return select(sql);
	}
	
	/**
	 * <br>執行 insert 。
	 * @param sql sql 語法。
	 * @return int insert 的結果， 0 => 沒有新增、1 => 新增一列成功、 -1 => 產生 Exception。
	 */
	public static int insert(String sql){
		return executeUpdate(sql);
	}
	
	/**
	 * <br>執行 insert 。
	 * @param tableName 資料表名稱。
	 * @param values 新增所需的欄位資料所組成的字串。
	 * @return int insert 的結果， 0 => 沒有新增、1 => 新增一列成功、 -1 => 產生 Exception。
	 */
	public static int insert(String tableName, String values){
		String sql = "insert into " + tableName +  values + ";";
		return executeUpdate(sql);
	}
	
	/**
	 * <br>執行 update 。
	 * @param sql sql 語法。
	 * @return int update 的結果， 0 => 沒有update、1 => 更新一列成功、 -1 => 產生 Exception。
	 */
	public static int update(String sql){
		return executeUpdate(sql);
	}
	
	/**
	 * <br>執行 delete 。
	 * @param sql sql 語法。
	 * @return int delete 的結果， 0 => 沒有delete、1 => delete 一列成功、 -1 => 產生 Exception。
	 */
	public static int delete(String sql){
		return executeUpdate(sql);
	}
	
	/**
	 * <br>執行 delete 。
	 * @param tableName 資料表名稱。
	 * @param conditions delete 所需的 where 條件。
	 * @return int delete 的結果， 0 => 沒有 delete、1 => delete 一列成功、 -1 => 產生 Exception。
	 */
	public static int delete(String tableName, String conditions){
		String sql = "delete from " + tableName + conditions + ";";
		return delete(sql);
	}
	
	/**
	 * <br>執行 insert 、 update 、 delete 的 sql。
	 * @param sql sql 語法。
	 * @return int 更動的結果， 0 => 沒有更動、1 => 更動一列成功、 -1 => 產生 Exception。
	 */
	protected static int executeUpdate(String sql){
		Statement st = DBConnector.getStatement();
		try {
			return st.executeUpdate(sql);
		} catch (SQLException e) {
			e.printStackTrace();
			return 0;
		} finally {
			try {
				DBConnector.close(st);
			} catch (SQLException e) {
				e.printStackTrace();
			}
		}
	}
	
	
	public static void close(){
		try {
			DBConnector.close(stList.get(stList.size()-1));
			stList.remove(stList.size()-1);
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}
	
	public static int getStListSize(){
		return stList.size();
	}

	/**
	 * 利用Spring執行 insert
	 * @param table 資料表
	 * @param colNames 欄位
	 * @param objects 要新增的內容
	 * @return	insert的結果。成功=>1
	 */
	public static int insert(String table, String[] colNames, Object[] objects) {
		JdbcTemplate jdbcTemplate = DBConnector.getJdbcTemplate();
		String sql, str;
		sql = "insert into "+ table;
		str = " ("+colNames[0];
		for(int i=1;i<colNames.length;i++){
			str += ", "+colNames[i];
		}
		str += ") ";
		sql += str;
		str = "values(null";
		for(int i=0;i<objects.length;i++){
			str += ", ?";
		}
		str += ");";
		sql += str;
		return jdbcTemplate.update(sql, objects);
	}

	/**
	 * 利用Spring執行 delete
	 * @param table 資料表
	 * @param strings 刪除需要等於的欄位
	 * @param objects 刪除需要等於的值
	 * @return	delete的結果。成功 = 1, 失敗 = -1
	 */
	public static int delete(String table, String[] strings, Object[] objects) {
		JdbcTemplate jdbcTemplate = DBConnector.getJdbcTemplate();
		String sql;
		if(strings.length == objects.length){
			sql = "delete from "+ table;
			sql += " where "+strings[0]+"=? ";
			for(int i=1;i<strings.length;i++){
				sql += "and "+strings[i]+"=? ";
			}
			sql += ";";
			return jdbcTemplate.update(sql, objects);
		}
		return -1;
	}
	
	/**
	 * 利用Spring執行 delete By ID
	 * @param table 資料表
	 * @param strings 刪除需要等於的欄位
	 * @param objects 刪除需要等於的值
	 * @return	delete的結果。成功 = 1, 失敗 = -1
	 */
	public static int delete(String table, int id) {
		JdbcTemplate jdbcTemplate = DBConnector.getJdbcTemplate();
		String sql;
		sql = "delete from "+ table;
		sql += " where id=? ;";
		return jdbcTemplate.update(sql, id);
	}

	/**
	 * 利用Spring執行 update
	 * @param table 資料表
	 * @param strings 要更新的欄位
	 * @param objects 要更新的值
	 * @param id where id = ?
	 * @return	update的結果。成功 = 1, 失敗 = -1
	 */
	public static int update(String table, String[] strings, Object[] objects, int id) {
		JdbcTemplate jdbcTemplate = DBConnector.getJdbcTemplate();
		String sql;
		if(strings.length == objects.length){
			sql = "update "+ table;
			sql += " set "+strings[0]+"=? ";
			for(int i=1;i<strings.length;i++){
				sql += ", "+strings[i]+"=? ";
			}
			sql += "where id="+ id +";";
			return jdbcTemplate.update(sql, objects);
		}
		return -1;
	}

	public static SqlRowSet select(String sql, Object... object) {
		JdbcTemplate jdbcTemplate = DBConnector.getJdbcTemplate();
		return jdbcTemplate.queryForRowSet(sql, object);
	}
}
