package tools.util;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontFormatException;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.geom.Rectangle2D;
import java.awt.print.PageFormat;
import java.awt.print.Printable;
import java.awt.print.PrinterException;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.Vector;

import javax.swing.JComponent;


public class FilePageRanderer extends JComponent implements Printable {

	/**
	 * 
	 */
	private static final long serialVersionUID = -1532259611061125926L;

	private int mCurrentPage;
	private Vector<String> mLines;
	private Vector<Vector<String>> mPages;
	private Font mFont;
	private int mFontSize;
	private Dimension mPreferredSize;
	
	public FilePageRanderer(File file, PageFormat pageFormat) throws IOException, FontFormatException {
		mFontSize = 12;
		
		mFont = new Font("Monospaced", Font.PLAIN, mFontSize);
		BufferedReader in = new BufferedReader(new FileReader(file));
		String line;
		mLines = new Vector<String>();
		int limit = (int)(pageFormat.getWidth() / mFontSize);
		while((line = in.readLine()) != null) {
			if(line.length() > limit) {
				int len = 0;
				
				while (len < line.length()) {
					if(len + limit - 1 >= line.length()) {
						mLines.addElement(line.substring(len, line.length()));
					} else {
						mLines.addElement(line.substring(len, len + limit - 1));
					}
					len += limit - 1;
				}
			} else {
				mLines.addElement(line);
			}
		}
		in.close();
		paginate(pageFormat);
	}
	
	public void paginate(PageFormat pageFormat) {
		mCurrentPage = 0;
		mPages = new Vector<Vector<String>>();
		float y = mFontSize;
		Vector<String> page = new Vector<String>();
		for (int i = 0; i < mLines.size(); i++) {
			String line = (String)mLines.elementAt(i);
			page.addElement(line);
			y += mFontSize;
			if (y + mFontSize * 2 > pageFormat.getImageableHeight()) {
				y = 0;
				mPages.addElement(page);
				page = new Vector<String>();
			}
		}
		
		if (page.size() > 0) {
			mPages.addElement(page);
		}
		
		mPreferredSize = new Dimension((int)pageFormat.getImageableWidth(), (int)pageFormat.getImageableHeight());
		repaint();
	}
	
	public void paintComponent(Graphics g) {
		Graphics2D g2d = (Graphics2D)g;
		Rectangle2D r2d = new Rectangle2D.Float(0, 0, mPreferredSize.width, mPreferredSize.height);
		g2d.setPaint(Color.white);
		g2d.fill(r2d);
		Vector<String> page = (Vector<String>)mPages.elementAt(mCurrentPage);
		g2d.setFont(mFont);
		g2d.setPaint(Color.black);
		float x = 0;
		float y = mFontSize;
		for (int i = 0; i < page.size(); i++) {
			String line = (String)page.elementAt(i);
			if (line.length() > 0) {
				g2d.drawString(line, (int)x, (int)y);
			}
			y += mFontSize;
		}
	}

	@Override
	public int print(Graphics g, PageFormat pageFormat, int pageIndex)
			throws PrinterException {
		if (pageIndex >= mPages.size()) {
			return NO_SUCH_PAGE;
		}
		
		int savePage = mCurrentPage;
		mCurrentPage = pageIndex;
		Graphics2D g2d = (Graphics2D)g;
		g2d.translate(pageFormat.getImageableX(), pageFormat.getImageableY());
		paint(g2d);
		mCurrentPage = savePage;
		return PAGE_EXISTS;
	}
	
	public Dimension getPreferredSize() {
		return mPreferredSize;
	}
	
	public int getCurrentPage() {
		return mCurrentPage;
	}
	
	public int getNunPages() {
		return mPages.size();
	}
	
	public void nextPage() {
		if (mCurrentPage < mPages.size() - 1) {
			mCurrentPage++;
		}
		repaint();
	}
	
	public void previousPage() {
		if (mCurrentPage > 0) {
			mCurrentPage--;
		}
		repaint();
	}
}
