package tools.util;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.print.PageFormat;
import java.awt.print.PrinterJob;
import java.io.File;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JMenuBar;
import javax.swing.JScrollPane;

import main.experimental.ui.innerQuestionPane.InnerQuestionPane;


public class FilePrinter extends JDialog implements ActionListener {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1983488272015885795L;
	private PageFormat mPageFormat;
	private FilePageRanderer mPageRanderer;
	private String mTitle;
	private JButton printBtn;
	private JButton nextPageBtn;
	private JButton previousPageBtn;
	private JButton closeBtn;
	private InnerQuestionPane iq;
	
	public FilePrinter(File file, InnerQuestionPane iq) {
		this.iq = iq;
		createUI();
		PrinterJob pj = PrinterJob.getPrinterJob();
		mPageFormat = pj.defaultPage();
		OpenFile(file);
		this.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
		this.pack();
		this.setModal(true);
		ScreenSet.setWidowCenter(this);
		this.setVisible(true);
	}

	private void createUI() {
		this.setSize(350, 300);
		
		Container content = this.getContentPane();
		content.setLayout(new BorderLayout());
		
		JMenuBar mb = new JMenuBar();
		
		printBtn = new JButton("Print");
		nextPageBtn = new JButton("Next page");
		previousPageBtn = new JButton("Previous page");
		closeBtn = new JButton("Close");
		
		printBtn.addActionListener(this);
		nextPageBtn.addActionListener(this);
		previousPageBtn.addActionListener(this);
		closeBtn.addActionListener(this);
		
		mb.add(printBtn);
		mb.add(nextPageBtn);
		mb.add(previousPageBtn);
		mb.add(closeBtn);
		
		this.setJMenuBar(mb);
		
		this.getContentPane().setLayout(new BorderLayout());
	}
	
	public void showTitle() {
		int currentPage = mPageRanderer.getCurrentPage() + 1;
		int numPages = mPageRanderer.getNunPages();
		this.setTitle(mTitle + " - page " + currentPage + " of " + numPages);
	}
	
	public void OpenFile(File file) {
		try {
			mPageRanderer = new FilePageRanderer(file, mPageFormat);
			mTitle = "[" + file.getName() + "]";
			showTitle();
			JScrollPane jsp = new JScrollPane(mPageRanderer);
			this.getContentPane().removeAll();
			this.getContentPane().add(jsp, BorderLayout.CENTER);
			this.validate();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	@Override
	public void actionPerformed(ActionEvent event) {
		Object btn = event.getSource();
		if(btn.equals(printBtn)){
			PrinterJob pj = PrinterJob.getPrinterJob();
			pj.setPrintable(mPageRanderer, mPageFormat);
			if (pj.printDialog()) {
				try {
					pj.print();
					iq.setPrint(true);
				} catch (Exception e) {
					e.printStackTrace();
					iq.setPrint(false);
				}
			}
			this.dispose();
		}
		if(btn.equals(nextPageBtn)){
			if (mPageRanderer != null) {
				mPageRanderer.nextPage();
			}
			showTitle();
		}
		if(btn.equals(previousPageBtn)){
			if (mPageRanderer != null) {
				mPageRanderer.previousPage();
			}
			showTitle();
		}
		if(btn.equals(closeBtn)){
			this.dispose();
		}
	}
}
