package tools.validation;

import org.springframework.validation.Errors;
import org.springframework.validation.ValidationUtils;
import org.springframework.validation.Validator;

import tools.util.Constant;
import vo.MailInfo;
import vo.MailSettingData;

public class MailSettingValidation implements Validator {
	
	public boolean supports(Class<?> arg0) {
		return MailInfo.class.isAssignableFrom(arg0);
	}

	public void validate(Object object, Errors errors) {
		MailSettingData info = (MailSettingData) object;
		ValidationUtils.rejectIfEmptyOrWhitespace(errors, "smtp", "required", "smtp:required");
		ValidationUtils.rejectIfEmptyOrWhitespace(errors, "name", "required", "name:required");
		ValidationUtils.rejectIfEmptyOrWhitespace(errors, "sender", "required", "sender:required");
		
		if(checkEmail(info.getSender())){
			errors.rejectValue("sender", "sender email format error", "sender email: format error");
		}
		
		//檢查 port 是否為數字。
		if(!"".equals(info.getPort())){
			try {
				Integer.parseInt(info.getPort());
			} catch (Exception e) {
				errors.rejectValue("port", "port must be a number", "port: format error");
			}
		}
		
		//檢查 mail template 是否包含置換 Pattern。
		if(!info.getContent().contains(Constant.LINK_REPLACEMENT)){
			errors.rejectValue("content", "content not contains " + Constant.LINK_REPLACEMENT, "content: not contains " + Constant.LINK_REPLACEMENT);
		}
	}
	
	/**
	 * 檢查 Email 的格式。
	 * @param mail
	 * @return boolean
	 */
	protected boolean checkEmail(String mail){
		return !mail.matches("^[_a-z0-9-]+([.][_a-z0-9-]+)*@[a-z0-9-]+([.][a-z0-9-]+)*$");
	}
}
